
/*
 *  Diverse Bristol audio routines.
 *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2006
 *
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdio.h>
#include <sys/types.h>

#include "brightoninternals.h"

static float opacity = 1.0;
static int shiftflag = 0;
static int controlflag = 0;
int logoid = -1;

brightonKeyPress(brightonWindow *bwin, brightonEvent *event)
{
	brightonIResource *panel;

	if (event->key == 'p')
		brightonXpmWrite(bwin, "/tmp/brighton.xpm");

	/*
	 * All of the following up to the dispatch is for the top layer opacity, it
	 * could go into a separate file?
	 *
	 * T should just be a toggle between totally opaque and the current 
	 * setting.
	 */
	if (event->key == 't')
	{
		float hold;

		if (logoid >= 0) {
			brightonRemove(bwin, logoid);
			logoid = -1;
		}

		hold = bwin->opacity;
		bwin->opacity = opacity;
		opacity = hold;
		brightonFinalRender(bwin, 0, 0, bwin->width, bwin->height);
	} else if ((event->key == 65508) || (event->key == 65509)) {
		controlflag = 1;
	} else if ((event->key == 65505) || (event->key == 65506)) {
		shiftflag = 1;
	} else if (event->key == 'o') {
		/*
		 * 'O' should be to make more opaque, 'o' more transparent. This is not
		 * as * trivial as it seems since the shift key is always a separate
		 * event.
		 */
		if (shiftflag)
		{
			if (bwin->opacity == 1.0f)
				bwin->opacity = 0.2f;
			else if ((bwin->opacity += 0.1f) > 1.0f)
				bwin->opacity = 1.0f;
			brightonFinalRender(bwin, 0, 0, bwin->width, bwin->height);
		} else {
			if (bwin->opacity <= 0.21f)
				bwin->opacity = 1.0f;
			else if ((bwin->opacity -= 0.2f) < 0.2f)
				bwin->opacity = 0.2f;
			brightonFinalRender(bwin, 0, 0, bwin->width, bwin->height);
		}
	} else if (event->key == 'l') {
		if (logoid < 0)
			logoid = brightonPlace(bwin, "bitmaps/images/bristol.xpm",
				bwin->width / 8, bwin->height / 2,
				bwin->width * 7 / 8, 50);
		else {
			brightonRemove(bwin, logoid);
			logoid = -1;
		}
	} else {
		if (logoid >= 0) {
			/*
			 * This attempts a fade out but would need to reset opacity and
			 * redraw to be correct, otherwise it taints the opacity.
			 *
			float opacity = bwin->opacity;

			bwin->opacity = bwin->opacity / 2;

			brightonFinalRender(bwin, 0, 0, bwin->width, bwin->height);

			brightonRemove(bwin, logoid);
			logoid = -1;

			bwin->opacity = opacity;
			*
			* So rather than all that just remove the watermarked logo
			*/

			brightonRemove(bwin, logoid);
			logoid = -1;
		}
	}

	if ((bwin->flags & BRIGHTON_DEV_ACTIVE) && (bwin->activepanel != 0))
		bwin->activepanel->configure(bwin, bwin->activepanel, event);
	else if ((panel = brightonPanelLocator(bwin, event->x, event->y)) > 0)
	{
		if (panel->configure)
			panel->configure(bwin, panel, event);
	}
}

brightonKeyRelease(brightonWindow *bwin, brightonEvent *event)
{
	brightonIResource *panel;

//	printf("brightonKeyRelease(%i)\n", event->key);

	if ((event->key == 65508) || (event->key == 65509))
		controlflag = 0;
	else if ((event->key == 65505) || (event->key == 65506))
		shiftflag = 0;

	if ((bwin->flags & BRIGHTON_DEV_ACTIVE) && (bwin->activepanel != 0))
		bwin->activepanel->configure(bwin, bwin->activepanel, event);
	else if ((panel = brightonPanelLocator(bwin, event->x, event->y)) > 0)
	{
		if (panel->configure)
			panel->configure(bwin, panel, event);
	}
}

brightonButtonPress(brightonWindow *bwin, brightonEvent *event)
{
	int dev;

	if (logoid >= 0) {
		/*
		 * This attempts a fade out but would need to reset opacity and redraw
		 * to be correct, otherwise it taints the opacity.
		 *
		float opacity = bwin->opacity;

		bwin->opacity = bwin->opacity / 2;

		brightonFinalRender(bwin, 0, 0, bwin->width, bwin->height);

		brightonRemove(bwin, logoid);
		logoid = -1;

		bwin->opacity = opacity;
		*
		* So rather than all that just remove the watermarked logo
		brightonRemove(bwin, logoid);
		logoid = -1;
		*/
	}

	/*
	 * We need to determine which device is under the selection, and force
	 * statefull delivery of events to that device for further motion.
	 */
	bwin->activepanel = 0;

	if ((bwin->activepanel =
		brightonPanelLocator(bwin, event->x, event->y)) > 0)
	{
		bwin->flags |= BRIGHTON_DEV_ACTIVE;

		event->command = BRIGHTON_BUTTONPRESS;

		if (bwin->activepanel->configure)
			bwin->activepanel->configure(bwin, bwin->activepanel, event);
	} else
		bwin->flags &= ~BRIGHTON_DEV_ACTIVE;
}

brightonButtonRelease(brightonWindow *bwin, brightonEvent *event)
{
//printf("brightonButtonRelease(%x, %x, %x)\n", bwin, bwin->activepanel,
//bwin->activepanel->configure);

	event->command = BRIGHTON_BUTTONRELEASE;

	//if (bwin->activepanel && bwin->activepanel->configure)
	if ((bwin->flags & BRIGHTON_DEV_ACTIVE) && (bwin->activepanel != 0))
		bwin->activepanel->configure(bwin, bwin->activepanel, event);

	bwin->flags &= ~BRIGHTON_DEV_ACTIVE;
	bwin->activepanel = 0;
}

brightonMotionNotify(brightonWindow *bwin, brightonEvent *event)
{
	if ((bwin->flags & BRIGHTON_DEV_ACTIVE) && (bwin->activepanel != 0))
	{
//printf("brightonMotionNotify(%x): %x %i/%i\n", bwin->activepanel,
//bwin->activepanel->configure, event->x, event->y);
		if (bwin->activepanel->configure)
			bwin->activepanel->configure(bwin, bwin->activepanel, event);
	}
}

brightonEnterNotify(brightonWindow *bwin, brightonEvent *event)
{
//	printf("brightonEnterNotify()\n");
}

brightonLeaveNotify(brightonWindow *bwin, brightonEvent *event)
{
//	printf("brightonLeaveNotify()\n");
}

brightonFocusIn(brightonWindow *bwin, brightonEvent *event)
{
//	printf("brightonFocusIn()\n");
}

brightonFocusOut(brightonWindow *bwin, brightonEvent *event)
{
//	printf("brightonFocusOut()\n");
}

brightonKeymapNotify(brightonWindow *bwin, brightonEvent *event)
{
	printf("brightonKeymapNotify()\n");
}

brightonExpose(brightonWindow *bwin, brightonEvent *event)
{
//	printf("brightonExpose(%i %i %i %i): %i\n",
//		event->x, event->y, event->w, event->h);

	BCopyArea(bwin->display, 
		event->x, event->y, event->w, event->h, event->x, event->y);
}

brightonGraphicsExpose(brightonWindow *bwin, brightonEvent *event)
{
//	printf("brightonGraphicsExpose()\n");
}

brightonNoExpose(brightonWindow *bwin, brightonEvent *event)
{
//	printf("brightonNoExpose()\n");
}

brightonVisibilityNotify(brightonWindow *bwin, brightonEvent *event)
{
//	printf("brightonVisibilityNotify()\n");
}

brightonCreateNotify(brightonWindow *bwin, brightonEvent *event)
{
//	printf("brightonCreateNotify()\n");
}

brightonDestroyNotify(brightonWindow *bwin, brightonEvent *event)
{
	brightonIResource *panel;

	/*
	 * Counter intuitive, we do nothing here. The process gets a SIGPIPE and
	 * the GUI intercepts this to request the engine to terminate the algo
	 * represented by this GUI. Engine will terminate when all algos have
	 * finished.
	 */
//	printf("brightonDestroyNotify()\n");
}

brightonUnmapNotify(brightonWindow *bwin, brightonEvent *event)
{
//	printf("brightonUnmapNotify()\n");
}

brightonMapNotify(brightonWindow *bwin, brightonEvent *event)
{
//	printf("brightonMapNotify()\n");
}

brightonMapRequest(brightonWindow *bwin, brightonEvent *event)
{
//	printf("brightonMapRequest()\n");
}

brightonReparentNotify(brightonWindow *bwin, brightonEvent *event)
{
//	printf("brightonReparentNotify()\n");
}

brightonConfigureNotify(brightonWindow *bwin, brightonEvent *event)
{
//	printf("brightonConfigureNotify(%i, %i, %i, %i)\n",
//		event->x, event->y, event->w, event->h);

	if (bwin->flags & BRIGHTON_SET_SIZE)
	{
		bwin->flags &= ~BRIGHTON_SET_SIZE;
		return;
	}

	if ((bwin->width != event->w) || (bwin->height != event->h))
		brightonWorldChanged(bwin, event->w, event->h);

	if ((bwin->width != event->w) || (bwin->height != event->h))
		BResizeWindow(bwin->display, bwin, bwin->width, bwin->height);
}

brightonConfigureRequest(brightonWindow *bwin, brightonEvent *event)
{
	printf("brightonConfigureRequest()\n");
}

brightonGravityNotify(brightonWindow *bwin, brightonEvent *event)
{
	printf("brightonGravityNotify()\n");
}

brightonResizeRequest(brightonWindow *bwin, brightonEvent *event)
{
//	printf("brightonResizeRequest()\n");

	if ((bwin->width != event->w) || (bwin->height != event->h))
	{
		/*
		 * Our size has changed. We need to re-render the background, and then 
		 * repaint it.
		 */
		brightonWorldChanged(bwin, event->w, event->h);

/*
		if ((bwin->width != event->w) || (bwin->height != event->h))
			BResizeWindow(bwin->display, bwin->win, bwin->width, bwin->height);
*/
	}
}

brightonCirculateNotify(brightonWindow *bwin, brightonEvent *event)
{
	printf("brightonCirculateNotify()\n");
}

brightonCirculateRequest(brightonWindow *bwin, brightonEvent *event)
{
	printf("brightonCirculateRequest()\n");
}

brightonPropertyNotify(brightonWindow *bwin, brightonEvent *event)
{
	printf("brightonPropertyNotify()\n");
}

brightonSelectionClear(brightonWindow *bwin, brightonEvent *event)
{
	printf("brightonSelectionClear()\n");
}

brightonSelectionRequest(brightonWindow *bwin, brightonEvent *event)
{
	printf("brightonSelectionRequest()\n");
}

brightonSelectionNotify(brightonWindow *bwin, brightonEvent *event)
{
	printf("brightonSelectionNotify()\n");
}

brightonColormapNotify(brightonWindow *bwin, brightonEvent *event)
{
	printf("brightonColormapNotify()\n");
}

brightonClientMessage(brightonWindow *bwin, brightonEvent *event)
{
	printf("brightonClientRequest()\n");
}

brightonMappingNotify(brightonWindow *bwin, brightonEvent *event)
{
	printf("brightonMappingNotify()\n");
}

brightonNullHandler(brightonWindow *bwin, brightonEvent *event)
{
	printf("brightonNullHandler()\n");
}

typedef int (*eventRoutine)(brightonWindow *, brightonEvent *);

/*
 * Check out the bEvent from the library, it can mask some events that are
 * considered "not important", but actually may be.
 */
eventRoutine defaultHandlers[BLASTEvent] = {
	brightonNullHandler,
	brightonNullHandler,
	brightonKeyPress,
	brightonKeyRelease,
	brightonButtonPress,
	brightonButtonRelease,
	brightonMotionNotify,
	brightonEnterNotify,
	brightonLeaveNotify,
	brightonFocusIn,
	brightonFocusOut,
	brightonKeymapNotify,
	brightonExpose,
	brightonGraphicsExpose,
	brightonNoExpose,
	brightonVisibilityNotify,
	brightonCreateNotify,
	brightonDestroyNotify,
	brightonUnmapNotify,
	brightonMapNotify,
	brightonMapRequest,
	brightonReparentNotify,
	brightonConfigureNotify,
	brightonConfigureRequest,
	brightonGravityNotify,
	brightonResizeRequest,
	brightonCirculateNotify,
	brightonCirculateRequest,
	brightonPropertyNotify,
	brightonSelectionClear,
	brightonSelectionRequest,
	brightonSelectionNotify,
	brightonColormapNotify,
	brightonClientMessage,
	brightonMappingNotify
};

void brightonInitDefHandlers(brightonWindow *bwin)
{
	int i;

	for (i = 0; i < BLASTEvent; i++)
		bwin->callbacks[i] = defaultHandlers[i];
}

void
brightonOldEventLoop(brightonDisplay **dlist)
{
	brightonDisplay *display;
	brightonEvent event;
	brightonWindow *bwin = (*dlist)->bwin;

	while (1) {
		/*
		 * BNextEvent will block due to it using XNextEvent. This makes the
		 * use of other functions rather difficult. Will rework this to use
		 * XCheckMaskEvent which will not block.
		 *
		 *	while (1) {
		 *		foreach (win) {
		 * 			BCheckMaskEvent(bwin, &event);
		 *		}
		 *	}
		 *
		 * We need this since the MIDI routines also need to be checked, so we
		 * will select on the ALSA SEQ port looking for controller changes so
		 * that they can modify the GUI display.
		 */
		BNextEvent(bwin->display, &event);

		if (event.command == BRIGHTON_NONE)
			continue;

		/*
		 * This may need to be changed to a semaphore, but I am hoping all GUI
		 * activity can occupy a single thread only.
		 */
		bwin->flags |= BRIGHTON_BUSY;

		/*
		 * Look for the right window.
		 */
		display = *dlist;

		while (display != 0)
		{
			if (event.wid == ((brightonWindow *) display->bwin)->win)
				break;

			if ((event.type == BRIGHTON_DESTROY)
				&& (((brightonWindow *) display->bwin)->parentwin == event.wid))
				break;

			display = display->next;
		}

		if ((display == 0 || (event.type < 0) || (event.type >= BLASTEvent)))
			continue;

		((brightonWindow *) display->bwin)->callbacks[event.type]
			(display->bwin, &event);

		bwin->flags &= ~BRIGHTON_BUSY;
	}
}

void
brightonEventLoop(brightonDisplay **dlist)
{
	brightonDisplay *display;
	brightonEvent event;
	brightonWindow *bwin = (*dlist)->bwin;

//	while (1) {
		/*
		 * BNextEvent will block due to it using XNextEvent. This makes the
		 * use of other functions rather difficult. Will rework this to use
		 * XCheckMaskEvent which will not block.
		 *
		 *	while (1) {
		 *		foreach (win) {
		 * 			BCheckMaskEvent(bwin, &event);
		 *		}
		 *	}
		 *
		 * We need this since the MIDI routines also need to be checked, so we
		 * will select on the ALSA SEQ port looking for controller changes so
		 * that they can modify the GUI display.
		 */
		while (BNextEvent(bwin->display, &event) > 0)
		{
			if (event.command == BRIGHTON_NONE)
				continue;

			/*
			 * This may need to be changed to a semaphore, but I am hoping all
			 * GUI activity can occupy a single thread only.
			 */
			bwin->flags |= BRIGHTON_BUSY;

			/*
			 * Look for the right window.
			 */
			display = *dlist;

			while (display != 0)
			{
				if (event.wid == ((brightonWindow *) display->bwin)->win)
					break;

				if ((event.type == BRIGHTON_DESTROY)
					&& (((brightonWindow *) display->bwin)->parentwin
						== event.wid))
					break;

				display = display->next;
			}

			if ((display == 0 || (event.type < 0)
				|| (event.type >= BLASTEvent)))
				continue;

			((brightonWindow *) display->bwin)->callbacks[event.type]
				(display->bwin, &event);

			bwin->flags &= ~BRIGHTON_BUSY;
		}

		/*
		 * This will now become a select on the ALSA SEQ socket looking for 
		 * MIDI events. Not certain how they will be linked into the GUI at
		 * the moment. For now this is just a sleep until the ALSA SEQ interface
		 * registration code has been finalised.
		 *
		 * What we will be looking for are events on a MIDI channel, we will
		 * look for that MIDI channel in our window lists. We want to respond
		 * to key events and reflect that in the GUI optionally, but not send
		 * the key events since the engine will handle all those. We also want 
		 * to look for controller values and have some configurable method to
		 * link those to our controls. Now this linkage will be done via the
		 * GUI, preferably, with the <Control><Middle Mouse><MIDI CC # motion>.
		 * Since the GUI handles this then we can dispatch events to another
		 * module that does the linkage. Need to be able to save and retrieve
		 * configurations - <Control><Middle> will go to this module, and all
		 * MIDI controller events as well, and it will make the linkage and
		 * dispatch the events.
		 * We should try and have a vkeydb type X event keymap.
		 */
//		usleep(10000);
//	}
}

