
/*
 *  Diverse Bristol audio routines.
 *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2006
 *
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdio.h>
#include <math.h>

/*
 * This is for some basic profiling of some of the routines, needed to optimise
 * the codebase.
 */
//#define STATS
#ifdef STATS
#include <sys/time.h>
struct timeval tstart, tend;
#endif

#include "brightoninternals.h"

#define SQRT_TAB_SIZE 128
double sqrttab[SQRT_TAB_SIZE][SQRT_TAB_SIZE];

initSqrtTab()
{
	int i, j, pindex;

	for (j = 0 ; j < SQRT_TAB_SIZE; j++)
	{
		for (i = 0 ; i < SQRT_TAB_SIZE; i++)
		{
			if ((sqrttab[i][j] = sqrt((double) (i * i + j * j))) == 0)
				sqrttab[i][j] = 0.0001;
		}
	}
}

int
brightonInitBitmap(brightonBitmap *bitmap, int value)
{
	register int index, total = bitmap->width * bitmap->height;

	for (index = 0; index < total; index++)
		bitmap->pixels[index] = value;
}

extern int logoid;

/*
 * This is dead slow.
 */
brightonWorldChanged(brightonWindow *bwin, int width, int height)
{
	float aspect;

printf("brightonWorldChanged(%i %i %i %i)\n", width, height,
bwin->width, bwin->height);
	/*
	 * We need to maintain aspect ratio. First take the new width, and build
	 * a corresponding height. If the height is outside the realms of the
	 * display then do the reverse. If, however, the width and height are
	 * within a reasonable amount of the desired aspect ratio, just use
	 * them.
	 */
	aspect = ((float) width) / height;

	if (bwin->height == height)
	{
		/*
		 * width has changed, calculate new height. If height is our max,
		 * for this screen, then let it be max, and calculate alternative
		 * width.
		 */
		if ((width / bwin->aspect)
			< ((brightonDisplay *) bwin->display)->height)
		{
			bwin->width = width;
			bwin->height = bwin->width / bwin->aspect;
		} else {
			bwin->height = ((brightonDisplay *) bwin->display)->height - 10;
			bwin->width = bwin->height * bwin->aspect;
		}
	} else {
		if ((height * bwin->aspect)
			< ((brightonDisplay *) bwin->display)->width)
		{
			bwin->height = height;
			bwin->width = height * bwin->aspect;
		} else {
			bwin->width = ((brightonDisplay *) bwin->display)->width - 10;
			bwin->height = bwin->width / bwin->aspect;
		}
	}
	if ((aspect / bwin->aspect > 0.98) && ((aspect / bwin->aspect) < 1.02))
	{
		bwin->width = width;
		bwin->height = height;
	} else {
		bwin->flags |= BRIGHTON_SET_SIZE;
	}

	brightonFreeBitmap(bwin, bwin->canvas);
	brightonFreeBitmap(bwin, bwin->render);
	brightonFreeBitmap(bwin, bwin->dlayer);
	brightonFreeBitmap(bwin, bwin->slayer);
	brightonFreeBitmap(bwin, bwin->tlayer);

	bwin->canvas = brightonCreateBitmap(bwin, bwin->width, bwin->height);
	bwin->dlayer = brightonCreateBitmap(bwin, bwin->width, bwin->height);
	bwin->slayer = brightonCreateBitmap(bwin, bwin->width, bwin->height);
	bwin->tlayer = brightonCreateBitmap(bwin, bwin->width, bwin->height);
	bwin->render = brightonCreateBitmap(bwin, bwin->width, bwin->height);

	if (bwin->opacity == 0)
		bwin->opacity = 0.5f;

	/*
	 * Our device and shadow layer need to be initialised with -1
	 */
	brightonInitBitmap(bwin->dlayer, -1);
	brightonInitBitmap(bwin->slayer, -1);
	brightonInitBitmap(bwin->tlayer, -1);

	if ((bwin->flags & BRIGHTON_NO_LOGO) == 0)
		logoid = brightonPlace(bwin, "bitmaps/images/bristol.xpm",
			bwin->width / 8, bwin->height / 2,
			bwin->width * 7 / 8, 50);

	bwin->lightX = -1000;
	bwin->lightY = -1000;
	bwin->lightH = 3000;
	bwin->lightI = 0.95;

	/*
	 * We now have a configured size. We need to render the surface onto the
	 * window background, and render the image if we have any configured 
	 * blueprints. These are rendered onto our canvas.
	 */
	if (bwin->app->flags & BRIGHTON_STRETCH)
		brightonStretch(bwin, bwin->surface, bwin->canvas, 0, 0,
			bwin->width, bwin->height, bwin->app->flags);
	else
		brightonTesselate(bwin, bwin->surface, bwin->canvas, 0, 0,
			bwin->width, bwin->height, bwin->app->flags);

	brightonStretch(bwin, bwin->image, bwin->canvas, 0, 0,
		bwin->width, bwin->height, 0);

	brightonRender(bwin, bwin->canvas, bwin->render, 0, 0,
		bwin->width, bwin->height, 0);

	bwin->flags |= BRIGHTON_NO_DRAW;

	if  (bwin->app)
	{
		int panel;
		brightonEvent event;

		for (panel = 0; panel < bwin->app->nresources; panel++)
		{
			/*
			 * We need to configure the device to resize itself
			 */
			event.command = BRIGHTON_RESIZE;
			event.x = (int) bwin->app->resources[panel].x
				* bwin->width / 1000;
			event.y = (int) bwin->app->resources[panel].y
				* bwin->height / 1000;
			event.w = (int) bwin->app->resources[panel].width
				* bwin->width / 1000;
			event.h = (int) bwin->app->resources[panel].height
				* bwin->height / 1000;

			bwin->app->resources[panel].configure(bwin,
				&bwin->app->resources[panel], &event);
		}
	}

	bwin->flags &= ~BRIGHTON_NO_DRAW;

	brightonFinalRender(bwin, 0, 0, bwin->width, bwin->height);

	/*
	 * Call the application to see if it wants to alter anything after the
	 * world changed.
	 */
	if (bwin->template->configure)
		bwin->template->configure(bwin);
}

brightonColorQuality(brightonWindow *bwin, float quality)
{
	if ((bwin->quality = quality) < 0.2f)
		bwin->quality = 0.2f;
	else if (bwin->quality > 1.0f)
		bwin->quality = 1.0f;
}

brightonOpacity(brightonWindow *bwin, float opacity)
{
	if ((bwin->opacity = opacity) < 0.01f)
		bwin->opacity = 0.01f;
	else if (bwin->opacity > 1.0f)
		bwin->opacity = 1.0f;
}

brightonLogo(brightonWindow *bwin)
{
	bwin->flags |= BRIGHTON_NO_LOGO;
}

static int
antialias(brightonWindow *bwin, brightonBitmap *b, int x, int y, float aa)
{
	register int nr, ng, nb, dy, color, cpre, cnext, pindex, cpy, cny;
	register brightonPalette *palette = bwin->display->palette;

	dy = x + y * bwin->width;

	cpy = b->pixels[x + (y - 1) * bwin->width];
	cny = b->pixels[x + (y + 1) * bwin->width];
	cpre = b->pixels[dy - 1];
	color = b->pixels[dy];
	cnext = b->pixels[dy + 1];

	/*
	 * Take a mix of the previous and next pixel.
	 */
	nr = palette[color].red / 2
		+ palette[cpy].red / 8 + palette[cny].red / 8
		+ palette[cpre].red / 8 + palette[cnext].red / 8;
	ng = palette[color].green / 2
		+ palette[cpy].green / 8 + palette[cny].green / 8
		+ palette[cpre].green / 8 + palette[cnext].green / 8;
	nb = palette[color].blue / 2
		+ palette[cpy].blue / 8 + palette[cny].blue / 8
		+ palette[cpre].blue / 8 + palette[cnext].blue / 8;
	/*
	nr = palette[color].red /2 + palette[cpre].red / 4
		+ palette[cnext].red / 4;
	ng = palette[color].green /2 + palette[cpre].green / 4
		+ palette[cnext].green / 4;
	nb = palette[color].blue /2 + palette[cpre].blue / 4
		+ palette[cnext].blue / 4;
	 */

	/*
	 * We want to be flexible on matches otherwise we use excessive colors and
	 * slow the GUI down
	 */
	if ((pindex = brightonFindColor(palette, bwin->cmap_size,
		nr, ng, nb, 0.95f)) >= 0)
	{
		palette[pindex].uses++;
		return(pindex);
	}

	/*
	 * If we have no free colors, then palette[0] is the default
	 */
	if ((pindex = brightonFindFreeColor(palette, bwin->cmap_size)) < 0)
		return(0);

	palette[pindex].red =  nr;
	palette[pindex].green = ng;
	palette[pindex].blue = nb;
	palette[pindex].uses++;

	palette[pindex].flags &= ~BRIGHTON_INACTIVE_COLOR;
	palette[pindex].uses++;

	return(pindex);
}

static int
makeTransparent(brightonWindow *bwin, int dest, int tlayer, float opacity)
{
	register int dr, dg, db, tr, tg, tb, nr, ng, nb;
	register int pindex;
	register brightonPalette *palette = bwin->display->palette;

	if (opacity >= 0.99)
		return(tlayer);

	/*
	 * Take the color selections between the two values, the more opaque the
	 * greater the more of the selected value will be taken from the top patch
	 * layer.
	 */
	nr = (int) (
		(((float) (palette[dest].red)) * (1.0f - opacity))
		+ (((float) (palette[tlayer].red)) * opacity));
	ng = (int) (
		(((float) (palette[dest].green)) * (1.0f - opacity))
		+ (((float) (palette[tlayer].green)) * opacity));
	nb = (int) (
		(((float) (palette[dest].blue)) * (1.0f - opacity))
		+ (((float) (palette[tlayer].blue)) * opacity));

	/*
	 * We want to be flexible on matches otherwise we use excessive colors and
	 * slow the GUI down
	 */
	if ((pindex = brightonFindColor(palette, bwin->cmap_size,
		nr, ng, nb, bwin->quality < 0.9f? bwin->quality:0.9f)) >= 0)
	{
		palette[pindex].uses++;
		return(pindex);
	}

	/*
	 * If we have no free colors, then palette[0] is the default
	 */
	if ((pindex =
		brightonFindFreeColor(palette, bwin->cmap_size)) < 0)
		return(0);

	palette[pindex].red =  nr;
	palette[pindex].green = ng;
	palette[pindex].blue = nb;
	palette[pindex].uses++;

	palette[pindex].flags &= ~BRIGHTON_INACTIVE_COLOR;
	palette[pindex].uses++;

	return(pindex);
}

/*
 * This is the final render algorithm.
 * It will take the window canvas, device layer and shadow layer at a given
 * offset, put them all onto the rendering plane, and update our screen 
 * interface library.
 */

#ifdef STATS
statFunction(int op, char *operation)
{
	if (op == 0)
		gettimeofday(&tstart, 0);
	else {
		gettimeofday(&tend, 0);
		/*
		 * I hate this calculation.
		 *
		 * If seconds are the same then take the difference of the micros.
		 * If seconds are not the same then diff secs - 1 blah blah blah
		 */
		if (tstart.tv_sec == tend.tv_sec)
			printf("%16s:  0.%-2i\n", operation, tend.tv_usec - tstart.tv_usec); 
		else {
			printf("%16s: %2i.%-2i\n", operation,
			tend.tv_sec - tstart.tv_sec - 1,
			1000000 - tstart.tv_usec + tend.tv_usec); 
		}
	}
}
#endif

brightonDoFinalRender(brightonWindow *bwin,
register int x, register int y, register int width, register int height)
{
	register int i, j, dy, index;
	register int *src = bwin->canvas->pixels;
	register int *dest = bwin->render->pixels;
	register int *device = bwin->dlayer->pixels;
	register int *shadow = bwin->slayer->pixels;
	register int *top = bwin->tlayer->pixels;

#ifdef STATS
	statFunction(0, "Brighton");
#endif

	for (j = y; j < (y + height); j++)
	{
		if (j >= bwin->height)
			break;

		dy = j * bwin->width;

		for (i = x; i < (x + width); i++)
		{
			if (i >= bwin->width)
				break;

			//index = i + j * bwin->width;
			index = i + dy;

			/*
			 * We have 3 layers, the topmost device layer, the middle shadow
			 * layer, and the lower canvas. The colors from each layer are
			 * moved into the render layer in order of priority. Where any
			 * layer has a negative brightonColor reference it is passed over
			 * to a lower plane. Other planes may be introduced at a later date
			 * as other features are incorporated.
			 *
			 * Adding another layer initially for patch cabling. We should
			 * consider adding the top layer last and allow it to act as a 
			 * transparancy, but that is non trivial since we are not dealing
			 * with colors, but with pixel identifiers. FFS.
			 *
			 * FS: We should paint in the pixel as if the transparency were
			 * not there, then, if we have a color in the transparency then we
			 * will paint it in with opacity.
			 *
			 * Going to try and add in some 'smoothing', or antialiasing of the
			 * rendering. Start with some trivial stuff - horizontal.
			 */
			if (device[index] >= 0)
				dest[i + dy] = device[index];
			else if (shadow[index] >= 0)
				dest[i + dy] = shadow[index];
			else {
				/*
				if (((i > 0) && (i < bwin->width - 1)) &&
					((j > 0) && (j < bwin->height - 1)))
					dest[i + dy] = antialias(bwin, bwin->canvas, i, j, 0.25);
				else
				*/
					dest[i + dy] = src[index];
			}

			if (top[index] >= 0)
				dest[i + dy] =
					makeTransparent(bwin, dest[i + dy], top[index],
						bwin->opacity);
		}
	}

#ifdef STATS
	statFunction(1, "Brighton");
	statFunction(0, "Library");
#endif

	/*
	 * Finally call the B library, of which currently only one version, the
	 * B11 interface to X11.
	 */
	BDrawArea(bwin->display, bwin->render, x, y, width, height, x, y);

#ifdef STATS
	statFunction(1, "Library");
#endif
}

brightonFinalRender(brightonWindow *bwin,
register int x, register int y, register int width, register int height)
{
	/* 
	 * Speed up on resizing - prevent excess redraws since we will have to
	 * scan the whole lot again anyway
	 */
	if (bwin->flags & BRIGHTON_NO_DRAW)
		return;

//	if (bwin->flags & BRIGHTON_BUSY)
//		return;

//	bwin->flags |= BRIGHTON_BUSY;

	brightonDoFinalRender(bwin, x, y, width, height);

//	bwin->flags &= ~BRIGHTON_BUSY;
}

/*
 * A tesselation algorithm, takes a bitmap and tiles it onto the specified area.
 */
brightonTesselate(register brightonWindow *bwin, register brightonBitmap *src,
register brightonBitmap *dest, register int x, register int y,
register int width, register int height, int direction)
{
	register int i, j, dy, w, h;
	register brightonPalette *palette = bwin->display->palette;
	register int *pixels;

	if ((src == 0) || (dest == 0))
		return;

//printf("brightonTesselate()\n");

	pixels = src->pixels;

	for (j = 0; j < height; j += src->height)
	{
		if (((j + y) >= dest->height) || (j >= height))
			break;

		dy = (j + y) * dest->width;

		/*
		 * Bounds check for reduction of last image to be rendered.
		 */
		if (j + src->height >= height)
			h = height - j;
		else
			h = src->height;

		for (i = 0; i < width; i += src->width)
		{
			if (((i + x) >= dest->width) || (i >= width))
				break;

			if (i + src->width >= width)
				w = width - i;
			else
				w = src->width;

			brightonRender(bwin, src, dest, i + x, j + y, w, h, direction);
		}
	}
}

/*
 * This is a render algorithm. Takes a bitmap and copies it to the
 * brightonBitmap area. We should do some bounds checking first?
 */
brightonRender(register brightonWindow *bwin, register brightonBitmap *src,
register brightonBitmap *dest, register int x, register int y,
register int width, register int height, int direction)
{
	register int i, j, dy;
	register int *pixels;

	if ((src == 0) || (dest == 0))
		return;

//printf("	render %i %i %i %i, %i %i, %i %i\n", x, y, width, height,
//src->width, src->height, dest->width, dest->height);

	pixels = src->pixels;

	for (j = 0; j < height; j++)
	{
		if (((j + y) >= dest->height) || (j >= height))
			break;

		dy = (j + y) * dest->width + x;

		for (i = 0; i < width; i++)
		{
			if (((i + x) >= dest->width) || (i >= width))
				break;

			dest->pixels[i + dy] = src->pixels[i + j * src->width];
		}
	}
}

/*
 * This is a scaling render algorithm. Takes a bitmap and speads it across
 * the brightonWindow canvas area. We should do some bounds checking first.
 */
brightonStretch(register brightonWindow *bwin, brightonBitmap *src,
register brightonBitmap *dest, register int x, register int y,
register int width, register int height, int direction)
{
	register float i, j, x1 = 0;
	register int y1 = 0, cindex, drawcount = 0, ix1;
	register brightonPalette *palette = bwin->display->palette;
	register int *pixels;

	if ((src == 0) || (dest == 0))
		return;

//printf("	stretch %i %i %i %i, %i %i: flags %x\n", x, y, width, height,
//src->width, src->height, direction);

	pixels = src->pixels;
	/*
	 * Do not support rendering outside of the panel area, excepting where we
	 * go over outer edges.
	 */
	if ((x < 0) || (x >= bwin->width) || (y < 0) || (y >= bwin->height))
		return(0);

	/*
	 * We now have the test bitmap, and it should have generated a palette.
	 * render the bits onto our background, and copy it over.
	 */
	for (i = y; i < (y + height); i++)
	{
		switch (direction & BRIGHTON_DIRECTION_MASK) {
			default:
			// Default case and reverse take same values here
			case BRIGHTON_REVERSE:
				y1 = (int) ((i - y) * src->height / height) * src->width;
				break;
			case BRIGHTON_VERTICAL:
				y1 = (i - y) / height * src->width;
				break;
			case (BRIGHTON_VERTICAL|BRIGHTON_REVERSE):
				if (direction & BRIGHTON_HALF_REVERSE)
				{
					y1 = (i - y) / height * src->width;
					if (y1 < src->width / 2)
						y1 += src->width / 2;
					else
						y1 -= src->width / 2;
				} else {
					y1 = src->width - (i - y) / height * src->width;
				}
				break;
		}

		for (j = x; j < (x + width); j++)
		{
			switch (direction & BRIGHTON_DIRECTION_MASK) {
				default:
				// Default case and reverse take same values here
				case 0:
					x1 = y1 + (j - x) * src->width / width;
					break;
				case BRIGHTON_REVERSE:
					/*
					 * This is for button operations, and is only "half"
					 * reversed. We should make this a rendering option?
					 */
					if (direction & BRIGHTON_HALF_REVERSE)
					{
						x1 = (j - x) / width * src->width;
						if (x1 < src->width / 2)
							x1 += y1 + src->width / 2;
						else
							x1 += y1 - src->width / 2;

						if (x1 >= src->width * src->height)
							x1 = src->width * src->height - 1;
					} else {
						x1 = y1 + src->width - (j - x) * src->width / width;
					}
					break;
				case BRIGHTON_VERTICAL:
				case (BRIGHTON_VERTICAL|BRIGHTON_REVERSE):
					x1 = y1
						+ (int) ((j - x) * src->height / width) * src->width;
					break;
			}

			ix1 = x1;

			/* 
			 * Set pixel. Since we are using brightonBitmaps we do not have to
			 * go around using X library calls to set the pixel states.
			 */
			if (!isblue(ix1, palette, pixels))
			{
				//printf("filling in %i from %i (%i %i %i)\n",
				//	(int) ((y + i) * dest->width + j + x), ix1,
				//	dest->width * dest->height, src->width * src->height, ix1);
				dest->pixels[(int) (i * dest->width + j)] = src->pixels[ix1];
			} else {
				/*
				 * If the pixel is blue then we should put in the bit from
				 * the current map?
				 */
			}
		}
	}
}

#define round(x) x - ((float) ((int) x)) >= 0.5? x, x + 1;

static int
brightonround(float x)
{
	int ix = x;
	float fx = ix;

	if (x - -fx >= 0.5)
		return(ix + 1);

	return(ix);
}

double roll = 0;
double rinc = 0.04;

/*
 * This could do with some XDrawPoints() optimisation as well.
 */
brightonRotate(register brightonWindow *bwin, register brightonBitmap *src,
register brightonBitmap *dest, register int dx, register int dy,
register int width, register int height, register double rotation)
{
	register int py, px;
	register double i, j;
	register int adjx, adjy, natx, naty;
	register double r, ho2, angle;
	register int x, y, ax, ay, index, cindex;
	register brightonPalette *palette = bwin->display->palette;
	register int *pixels;

//printf("brightonRotate(%i, %i, %i, %i)\n",
//dx, dy, src->width, src->height);

	if ((src == 0) || (dest == 0))
		return;

	pixels = src->pixels;
	ho2 = src->height / 2;
	/*
	 * Do not support rendering outside of the panel area, excepting where we
	 * go over outer edges.
	 */
	if ((dx < 0) || (dx >= bwin->width) || (dy < 0) || (dy >= bwin->height))
	{
		printf("bounds fail\n");
		return(0);
	}

	if (width & 0x01)
		width--;
	if (height & 0x01)
		height--;

	/*
	 * This is for 'wobble' on the pot cap as it rotates.
	 */
	if ((roll += rinc) > 0.2)
		rinc = -rinc;
	else if (roll < 0)
	{
		rinc = -rinc;
		roll = 0;
	}

	/*
	 * We now have the test bitmap, and it should have generated a palette.
	 * render the bits onto our background, and copy it over.
	 */
	for (py = 0; py < height; py+=1)
	{
		j = py * src->height / height;
		naty = j - ho2;

		if (py >= dest->height)
			break;

		for (px = 0; px < width; px++)
		{
			i = px * src->width / width;
			natx = i - ho2;

			if ((r = sqrttab[natx < 0?-natx:natx][naty < 0?-naty: naty]) > ho2)
				continue;

			/*
			 * Rotate this point, and then put them back into native coords.
			 * We need our own rounding algorithm - otherwise we get truncation.
			 */
			if (r < src->istatic) {
				/*
				 * Want to do something here to make the centre 'wobble'
				x = brightonround(natx + ho2);
				y = brightonround(naty + ho2);
				 */
				/*
				 * Take the angle of this point to the origin.
				 */
				if (naty < 0.0) {
					//angle = asin(natx / r) + rotation;
					angle = asin(natx / r) + 2*M_PI - roll;
					adjx = r * sin(angle);
					adjy = -r * cos(angle);
				} else {
					//angle = -asin(natx / r) + M_PI;
					angle = -asin(natx / r) + 2*M_PI - roll;
					adjx = -r * sin(angle);
					adjy = r * cos(angle);
				}
				x = brightonround(adjx + ho2);
				y = brightonround(adjy + ho2);

				//x = brightonround(natx + ho2);
				//y = brightonround(naty + ho2);
			} else if (r < src->ostatic) {
				/*
				 * Take the angle of this point to the origin.
				 */
				if (naty < 0.0) {
					angle = asin(natx / r) + rotation;
					adjx = r * sin(angle);
					adjy = -r * cos(angle);
				} else {
					angle = -asin(natx / r) + rotation;
					adjx = -r * sin(angle);
					adjy = r * cos(angle);
				}
				x = brightonround(adjx + ho2);
				y = brightonround(adjy + ho2);
			} else {
				x = brightonround(natx + ho2);
				y = brightonround(naty + ho2);
			}

			if ((x < 0) || (x >= src->height) || (y < 0) || (y >= src->height))
				continue;
			index = y * src->width + x;

			dest->pixels[(int) (((dy + py) * dest->width) + dx + px)]
				= src->pixels[(int) (y * src->width + x)];
		}
	}
}

