
/*
 *  Diverse Bristol audio routines.
 *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2006
 *
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */


#include <signal.h>
#include <stdio.h>
#include <stdlib.h>

#include <pthread.h>

#include "brighton.h"
#include "brightonMini.h"
#include "bristolhelp.h"

extern brightonApp miniApp;
extern brightonApp prophetApp;
extern brightonApp hammondApp;
extern brightonApp junoApp;
extern brightonApp dxApp;
extern brightonApp explorerApp;
extern brightonApp mixApp;
extern brightonApp hammondB3App;
extern brightonApp voxApp;
extern brightonApp rhodesApp;
extern brightonApp rhodesBassApp;
extern brightonApp pro10App;
extern brightonApp prophet52App;
extern brightonApp obxApp;
extern brightonApp obxaApp;
extern brightonApp polyApp;
extern brightonApp poly6App;
extern brightonApp axxeApp;
extern brightonApp odysseyApp;
extern brightonApp memMoogApp;
extern brightonApp arp2600App;

char *bristolhome = NULL;

guimain global;

brightonApp *synthesisers[32];

void *eventMgr();
int opacity = 50;
int quality = 95;
int logo = 0;

extern brightonMidiInput(bristolMidiMsg *);

/*
 * Need to make this multithreaded?
 */
int
main(int argc, char **argv)
{
	int argCount = 1, midiHandle = -1, midiFD = -1, i;
	pthread_t thread;
	bristolMidiMsg msg;

	signal(SIGINT, cleanupBristol);
	signal(SIGPIPE, cleanupBristolQuietly);
	signal(SIGHUP, cleanupBristol);

	global.home = getenv("BRISTOL");

	global.synths = (guiSynth *) brightonmalloc(sizeof(guiSynth));

	global.voices = 16;
	global.synths->voices = 16;
	global.synths->synthtype = BRISTOL_2600;
	global.port = -1;

	synthesisers[BRISTOL_MINI] = &miniApp;
	synthesisers[BRISTOL_PROPHET] = &prophetApp;
	synthesisers[BRISTOL_PRO52] = &prophet52App;
	synthesisers[BRISTOL_HAMMOND] = &hammondApp;
	synthesisers[BRISTOL_JUNO] = &junoApp;
	synthesisers[BRISTOL_DX] = &dxApp;
	synthesisers[BRISTOL_EXPLORER] = &explorerApp;
	synthesisers[BRISTOL_HAMMONDB3] = &hammondB3App;
	synthesisers[BRISTOL_VOX] = &voxApp;
	synthesisers[BRISTOL_RHODES] = &rhodesApp;
	synthesisers[BRISTOL_RHODES_BASS] = &rhodesBassApp;
	synthesisers[BRISTOL_PROPHET10] = &pro10App;
	synthesisers[BRISTOL_MIXER] = &mixApp;
	synthesisers[BRISTOL_OBX] = &obxApp;
	synthesisers[BRISTOL_OBXA] = &obxaApp;
	synthesisers[BRISTOL_POLY] = &polyApp;
	synthesisers[BRISTOL_POLY6] = &poly6App;
	synthesisers[BRISTOL_AXXE] = &axxeApp;
	synthesisers[BRISTOL_ODYSSEY] = &odysseyApp;
	synthesisers[BRISTOL_MEMMOOG] = &memMoogApp;
	synthesisers[BRISTOL_2600] = &arp2600App;

	global.synths->location = 0;

	/*
	 * close our standard input, create a pipe which will never be used.
	 */
	while (argc > argCount)
	{
		if ((strcmp(argv[argCount], "--help") == 0)
			|| (strcmp(argv[argCount], "-help") == 0)
			|| (strcmp(argv[argCount], "-h") == 0)
			|| (strcmp(argv[argCount], "--h") == 0))
		{
			if (BRISTOL_BUILD == 0)
				printf("bristol %i.%i.%i: ",
					BRISTOL_MAJOR, BRISTOL_MINOR, BRISTOL_PATCH);
			else
				printf("bristol %i.%i.%i-%i: ",
					BRISTOL_MAJOR, BRISTOL_MINOR, BRISTOL_PATCH, BRISTOL_BUILD);
			printf("%s", helptext);
			exit(0);
		}

		if ((strcmp(argv[argCount], "-V") == 0)
			&& (strlen(argv[argCount]) == 2))
		{
			if (BRISTOL_BUILD == 0)
				printf("bristol %i.%i.%i\n",
					BRISTOL_MAJOR, BRISTOL_MINOR, BRISTOL_PATCH);
			else
				printf("bristol %i.%i.%i-%i\n",
					BRISTOL_MAJOR, BRISTOL_MINOR, BRISTOL_PATCH, BRISTOL_BUILD);
			exit(0);
		}

		if (strcmp(argv[argCount], "-libtest") == 0)
			global.libtest = 1;

		if (strcmp(argv[argCount], "-logo") == 0)
			logo = 1;

		if ((strcmp(argv[argCount], "-quality") == 0) && (argCount < argc))
		{
			if ((quality = atoi(argv[argCount + 1])) < 20)
				quality = 20;
			else if (quality > 100)
				quality = 100;
			argCount++;
			continue;
		}

		if ((strcmp(argv[argCount], "-opacity") == 0) && (argCount < argc))
		{
			if ((opacity = atoi(argv[argCount + 1])) < 20)
				opacity = 20;
			else if (opacity > 100)
				opacity = 100;
			argCount++;
			continue;
		}

		if ((strcmp(argv[argCount], "-gain") == 0) && (argCount < argc))
		{
			if (argCount < argc)
				global.synths->gain = atoi(argv[argCount + 1]);
			argCount++;
		}

		if ((strcmp(argv[argCount], "-load") == 0) && (argCount < argc))
		{
			if (argCount < argc)
				global.synths->location = atoi(argv[argCount + 1]);
			argCount++;
		}

		if ((strcmp(argv[argCount], "-voices") == 0) && (argCount < argc))
		{
			if (argCount < argc)
				global.synths->voices = atoi(argv[argCount + 1]);
			argCount++;
		}

		if ((strcmp(argv[argCount], "-channel") == 0) && (argCount < argc))
		{
			if (argCount < argc)
				global.synths->midichannel = atoi(argv[argCount + 1]) - 1;
			argCount++;
		}

		if (strcmp(argv[argCount], "-engine") == 0)
			global.flags |= BRIGHTON_NOENGINE;

		if (strcmp(argv[argCount], "-port") == 0)
		{
			if (argCount < argc)
				global.port = atoi(argv[++argCount]);
		}

		/*
		 * And finally all the different synths
		 */
		if (strcmp(argv[argCount], "-mini") == 0)
			global.synths->synthtype = BRISTOL_MINI;

		if (strcmp(argv[argCount], "-hammond") == 0)
			global.synths->synthtype = BRISTOL_HAMMOND;

		if (strcmp(argv[argCount], "-prophet10") == 0)
			global.synths->synthtype = BRISTOL_PROPHET;

		if (strcmp(argv[argCount], "-prophet") == 0)
			global.synths->synthtype = BRISTOL_PROPHET;

		if (strcmp(argv[argCount], "-prophet10") == 0)
			global.synths->synthtype = BRISTOL_PROPHET;

		if (strcmp(argv[argCount], "-pro5") == 0)
			global.synths->synthtype = BRISTOL_PROPHET;

		if (strcmp(argv[argCount], "-pro10") == 0)
			global.synths->synthtype = BRISTOL_PROPHET10;

		if (strcmp(argv[argCount], "-dx") == 0)
			global.synths->synthtype = BRISTOL_DX;

		if (strcmp(argv[argCount], "-juno") == 0)
			global.synths->synthtype = BRISTOL_JUNO;

		if (strcmp(argv[argCount], "-sampler") == 0)
			global.synths->synthtype = BRISTOL_SAMPLER;

		if (strcmp(argv[argCount], "-bristol") == 0)
			global.synths->synthtype = BRISTOL_BRISTOL;

		if (strcmp(argv[argCount], "-voyager") == 0)
			global.synths->synthtype = BRISTOL_EXPLORER;

		if (strcmp(argv[argCount], "-explorer") == 0)
			global.synths->synthtype = BRISTOL_EXPLORER;
		if (strcmp(argv[argCount], "-voyager") == 0)
			global.synths->synthtype = BRISTOL_EXPLORER;

		if (strcmp(argv[argCount], "-mixer") == 0)
		{
			global.libtest = 1;
			global.synths->synthtype = BRISTOL_MIXER;
		}

		if (strcmp(argv[argCount], "-rhodesbass") == 0)
			global.synths->synthtype = BRISTOL_RHODES_BASS;
		else if (strcmp(argv[argCount], "-rhodes") == 0)
			global.synths->synthtype = BRISTOL_RHODES;

		if (strcmp(argv[argCount], "-vox") == 0)
			global.synths->synthtype = BRISTOL_VOX;

		if (strcmp(argv[argCount], "-ddd") == 0)
			global.synths->synthtype = BRISTOL_DDD;

		if (strcmp(argv[argCount], "-pro52") == 0)
			global.synths->synthtype = BRISTOL_PRO52;

		if (strcmp(argv[argCount], "-b3") == 0)
			global.synths->synthtype = BRISTOL_HAMMONDB3;

		if (strcmp(argv[argCount], "-obxa") == 0)
			global.synths->synthtype = BRISTOL_OBXA;
		else if (strcmp(argv[argCount], "-obx") == 0)
			global.synths->synthtype = BRISTOL_OBX;

		if (strcmp(argv[argCount], "-grok") == 0)
			global.synths->synthtype = BRISTOL_POLY;
		if (strcmp(argv[argCount], "-moog") == 0)
			global.synths->synthtype = BRISTOL_MINI;
		if (strcmp(argv[argCount], "-arp") == 0)
			global.synths->synthtype = BRISTOL_2600;
		if (strcmp(argv[argCount], "-roland") == 0)
			global.synths->synthtype = BRISTOL_JUNO;
		if (strcmp(argv[argCount], "-oberheim") == 0)
			global.synths->synthtype = BRISTOL_OBXA;

		if (strcmp(argv[argCount], "-ylop") == 0)
			global.synths->synthtype = BRISTOL_POLY6;

		if (strcmp(argv[argCount], "-onom") == 0)
			global.synths->synthtype = BRISTOL_POLY;

		if (strcmp(argv[argCount], "-arp2600") == 0)
			global.synths->synthtype = BRISTOL_2600;
		if (strcmp(argv[argCount], "-2600") == 0)
			global.synths->synthtype = BRISTOL_2600;

		if (strcmp(argv[argCount], "-axxe") == 0)
			global.synths->synthtype = BRISTOL_AXXE;

		if (strcmp(argv[argCount], "-odyssey") == 0)
			global.synths->synthtype = BRISTOL_ODYSSEY;

		if ((strcmp(argv[argCount], "-memory") == 0)
			|| (strcmp(argv[argCount], "-memmoog") == 0)
			|| (strcmp(argv[argCount], "-memorymoog") == 0))
			global.synths->synthtype = BRISTOL_MEMMOOG;

		argCount++;
	}

	if (synthesisers[global.synths->synthtype] == 0)
		exit(0);

	global.synths->resources = synthesisers[global.synths->synthtype];
	/*
	 * win is actually set by the configuration routines, but we use it 
	 * here anyway.
	 */
	global.synths->win =
		brightonInterface(synthesisers[global.synths->synthtype],
			((float) quality) / 100.0f);

	if (logo)
		brightonLogo(global.synths->win);

	/*
	 * win is actually set by the configuration routines, but we use it 
	 * here anyway.
	global.flags |= BRIGHTON_NOENGINE;
	global.synths->next = (guiSynth *) brightonmalloc(sizeof(guiSynth));
	global.synths->next->last = global.synths;
	global.synths->next->voices = 16;
	global.synths->next->midichannel = 2;
	global.synths->next->resources = synthesisers[BRISTOL_JUNO];
	global.synths->next->synthtype = BRISTOL_JUNO;
	global.synths->next->win = (int)
		brightonInterface(synthesisers[BRISTOL_JUNO]);
printf("	PAIR IS %x %x, %x %x\n", global.synths, global.synths->next,
global.synths->win, global.synths->next->win);
	 */

	/*
	 * Finally go and let the event manager handle our interface. Going to 
	 * create a separate GUI thread, which will allow us to handle things like
	 * MIDI events from the engine, timed operations, etc, from there.
	 */
	if (pthread_create(&thread, NULL, eventMgr, NULL) != 0)
		printf("Could not create GUI thread\n");
	if (pthread_detach(thread) != 0)
		printf("Could not detach GUI thread\n");

	brightonOpacity(global.synths->win, ((float) opacity) / 100.0f);

	/*
	 * Here we should open an ALSA SEQ interface. We want to get an index back
	 * and pass that to the event handler for selection.
	 */
	if (global.libtest != 1)
	{
		if ((midiHandle = bristolMidiOpen("0.0",
			BRISTOL_CONN_SEQ|BRISTOL_DUPLEX,
			-1, BRISTOL_REQ_NSX, brightonMidiInput, &global)) < 0)
		{
			printf("Error opening midi device %s\n", "0.0");
		}

		midiFD = bristolGetMidiFD(midiHandle);

		printf("opened GUI midi handle: %i, fd %i\n", midiHandle, midiFD);

		brightonReadConfiguration(global.synths->win,
			synthesisers[global.synths->synthtype],
			global.synths->midichannel,
			synthesisers[global.synths->synthtype]->name);
	}

	/*
	 * These should be synth specific?
	 */
	for (i = 0; i < 128; i++)
		global.synths->win->midimap[i] = i;

	while (1) {
		brightonEventMgr();

		/*
		 * This will now become a select on the ALSA SEQ socket looking for
		 * MIDI events. Not certain how they will be linked into the GUI at
		 * the moment. For now this is just a sleep until the ALSA SEQ interface
		 * registration code has been finalised.
		 *
		 * What we will be looking for are events on a MIDI channel, we will
		 * look for that MIDI channel in our window lists. We want to respond
		 * to key events and reflect that in the GUI optionally, but not send
		 * the key events since the engine will handle all those. We also want
		 * to look for controller values and have some configurable method to
		 * link those to our controls. Now this linkage will be done via the
		 * GUI, preferably, with the <Control><Middle Mouse><MIDI CC # motion>.
		 * Since the GUI handles this then we can dispatch events to another
		 * module that does the linkage. Need to be able to save and retrieve
		 * configurations - <Control><Middle> will go to this module, and all
		 * MIDI controller events as well, and it will make the linkage and
		 * dispatch the events.
		 *
		 * We should try and have a vkeydb type X event keymap.
		 */
		if (midiHandle >= 0)
			bristolMidiDevRead(midiHandle, &msg);

		usleep(10000);
	}
}

extern int vuInterval;

void *
eventMgr()
{
	while (1)
	{
		/*
		 * Do whatever we want. Will turn into a wait routine on the MIDI
		 * channel
		 */
		if (vuInterval != 0)
		{
			usleep(vuInterval);
			if (global.synths->flags & OPERATIONAL)
				doAlarm();
		}
		else
			sleep(1);
	}
}


cleanout(void *id)
{
	if (id)
		brightonRemoveInterface(id);
	cleanupBristol();
	exit(0);
}


clearout(int result)
{
	exit(result);
}

