
/*
 *  Diverse Bristol audio routines.
 *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2006
 *
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <string.h>

#include "brightoninternals.h"
#include "brightonMini.h"
#include "brightondev.h"

static int confflag = 0;
static brightonDevice *confdev[BRIGHTON_GANG_COUNT];
static float confval[BRIGHTON_GANG_COUNT];

extern guimain global;

char *pheader = "#\n\
# This the the MIDI controller profile, it keeps controller remappings for\n\
# converting one controller ID to another, and then controls which controllers\n\
# are tracked by with GUI devices. It may be edited manually, in which case\n\
# changes are maintained, and the GUI can also alter the controller tracking\n\
# by typing <Control><Middle Mouse Button>, the moving the desired control\n\
#\n\
# Remap format is \"CM: MidiCC   MidiCC\"\n\
# Control format is \"CC: MidiCC   panel/index [value]\"\n\
#\n\
# The values are integers from 0 to 16384, the fine resolution controller\n\
# value for the throw of this controller.\n\
#\n\
#\n";

/*
 * A device registration per controller. Needs more information to allow it
 * to be saved to a file though. There is an entry per midi channel.
static brightonDevice *devices[128][16];
 */

	/*
	 * These should be synth specific?
	for (j = 0; j < 16; j++)
		for (i = 0; i < 128; i++)
			bmidi.mapping[i][j] = i;
	 */

/*
 * This will take the specified messages and recode the GM2 portion for things
 * such as coarse/fine controllers. It will also take the values of the coded
 * values and turn them into a floating point value.
 *
 * This stuff is painful for several reasons - some GM2 values denote coarse
 * and fine, others denote semitone and cents, etc, some are integer rather
 * than floats. It might be better to call this later if parsing is required.
 *
 * We also need to have a midi controller mapping system. If a controller is
 * remapped then we should consider it to be coarse only?
 */
bristolMidiToGM2(brightonWindow *bwin, bristolMidiMsg *msg)
{
	if (msg->command != MIDI_CONTROL)
	{
		/*
		 * This may be counter productive?
		 */
		msg->GM2.c_id = 0;
		msg->GM2.value = 0;
		return;
	}

//printf("MidiToGM2: %i\n", msg->params.controller.c_id);
	/*
	 * Apply the mappings already if any are defined
	 */
	msg->params.controller.c_val = bwin->midimap[msg->params.controller.c_val];

	/*
	 * Keep a copy of the controller setting for later use.
	 */
	bwin->GM2values[msg->params.controller.c_id] = msg->params.controller.c_val;

	/*
	 * This are true for all the controllers, but the later ones may have
	 * specific modifications
	 */
	msg->GM2.c_id = msg->params.controller.c_id;
	msg->GM2.value = ((float) msg->params.controller.c_val) / 127.0f;
	msg->GM2.intvalue = msg->params.controller.c_val;
	msg->GM2.coarse = msg->params.controller.c_val;
	msg->GM2.fine = 0;

	/*
	 * These are coarse of the course/fine controls
	 */
	if (msg->params.controller.c_id < 14)
	{
		msg->GM2.coarse = msg->params.controller.c_val;
		msg->GM2.fine = bwin->GM2values[msg->params.controller.c_id + 32];
		msg->GM2.intvalue = msg->GM2.coarse << 7 + msg->GM2.fine;
		msg->GM2.value = ((float) msg->GM2.intvalue) / 16383.0f;
		return;
	}

	if (msg->params.controller.c_id < 32)
		return;

	/*
	 * Fine controls for the first 14 coarse/fine controllers.
	 */
	if (msg->params.controller.c_id < 46)
	{
		msg->GM2.c_id -= 32;
		msg->GM2.fine = msg->params.controller.c_val;
		msg->GM2.coarse = bwin->GM2values[msg->params.controller.c_id - 32];
		msg->GM2.intvalue = msg->GM2.coarse << 7 + msg->GM2.fine;
		msg->GM2.value = ((float) msg->GM2.intvalue) / 16383.0f;
		return;
	}

	if (msg->params.controller.c_id < 80)
		return;

	/*
	 * These are 4 buttons.
	if (msg->params.controller.c_id < 84)
	{
		if (msg->GM2.intvalue > 63)
			msg->GM2.value = 1.0;
		else
			msg->GM2.value = 0.0;
		return;
	}
	 */

	if (msg->params.controller.c_id < 96)
		return;

	/*
	 * These are 2 data entry buttons.
	if (msg->params.controller.c_id < 98)
	{
		if (msg->GM2.intvalue > 63)
			msg->GM2.value = 1.0;
		else
			msg->GM2.value = 0.0;
		return;
	}
	 */

	/*
	 * Non Registered Parameter Numbers.
	 */
	if ((msg->params.controller.c_id == 98)
		|| (msg->params.controller.c_id == 99))
	{
		msg->GM2.c_id == 99;
		msg->GM2.fine = bwin->GM2values[98];
		msg->GM2.coarse = bwin->GM2values[99];
		msg->GM2.intvalue = msg->GM2.coarse << 7 + msg->GM2.fine;
		msg->GM2.value = ((float) msg->GM2.intvalue) / 16383.0f;
		return;
	}

	/*
	 * Registered Parameter Numbers.
	 */
	if ((msg->params.controller.c_id == 100)
		|| (msg->params.controller.c_id == 101))
	{
		msg->GM2.c_id == 101;
		msg->GM2.fine = bwin->GM2values[100];
		msg->GM2.coarse = bwin->GM2values[101];
		msg->GM2.intvalue = msg->GM2.coarse << 7 + msg->GM2.fine;
		msg->GM2.value = ((float) msg->GM2.intvalue) / 16383.0f;
		return;
	}

	/*
	 * local Keyboard
	if (msg->params.controller.c_id == 122)
	{
		if (msg->GM2.intvalue > 63)
			msg->GM2.value = 1.0;
		else
			msg->GM2.value = 0.0;
		return;
	}
	 */
}

static int
brightonMapControl(brightonWindow *bwin, brightonApp *app,
int channel, char *param)
{
	int from, to;

	from = atoi(param);

	if ((param = index(param, ' ')) == NULL)
	{
		if ((param = index(param, 9)) == NULL)
		{
//			printf("failed to get target\n");
			return;
		}
	}
	to = atoi(param);

	if ((from < 0) || (from > 127)
		|| (to < 0) || (to > 127))
		return;

	bwin->midimap[from] = to;

	printf("Mapped %i to %i\n", from, to);
}

static int
brightonSetControl(brightonWindow *bwin, brightonApp *app,
int channel, char *param)
{
	int cc, panel, ind, i, j;
	char *pp;
	brightonIResource *p;
	float value = 0.2, maxvalue = 0.0;

	cc = atoi(param);

	if ((pp = index(param, ' ')) == NULL)
	{
		if ((pp = index(param, 9)) == NULL)
		{
//			printf("failed to get panel\n");
			return;
		}
	}
	panel = atoi(pp);

	if ((pp = index(param, '/')) == NULL)
	{
//		printf("failed to get index\n");
		return;
	}
	ind = atoi(++pp);

	if ((pp = index(param, '/')) == NULL)
	{
//		printf("failed to get index\n");
		return;
	}
	ind = atoi(++pp);

	if ((index(pp, ' ')) == NULL)
	{
		if ((pp = index(pp, 9)) == NULL)
		{
			printf("failed to get value\n");
			value = 1.0;
		} else {
			printf("got number: %s\n", pp);
			value = ((float) atoi(pp)) / 16384.0;
		}
	} else {
		pp = index(pp, ' ');
		value = ((float) atoi(pp)) / 16384.0;
	}

//printf("%i %i of max %i %i\n", panel, ind, app->nresources,
//app->resources[panel].ndevices);

	/*
	 * We need some error checking
	 */
	if ((cc < 0) || (cc > 127)
		|| (channel < 0) || (channel >= 16)
		|| (panel < 0) || (panel >= app->nresources)
		|| (ind < 0) || (ind >= app->resources[panel].ndevices))
		return;
	/*
	 * We now have what looks to be a valid tuple cc/panel/index. Find
	 * the configuration code for that device and plug it into our
	 * dispatcher.
	devices[cc][channel] = bwin->app->resources[panel].devlocn[ind].dev;
	 */
	p = (brightonIResource *) &bwin->app->resources[panel];
	/*
	 * This should not be indexed by the controller rather the device. That
	 * way we can gang controls. Since the only data we have is the cc and
	 * a device, we should limit the number of calls to perhaps 8?
	 */
	for (i = 0; i < BRIGHTON_GANG_COUNT; i++)
	{
		if (bwin->midicontrol[cc][i] == NULL)
		{
			bwin->midicontrol[cc][i]
				= ((struct brightonDevice *) p->devlocn[ind].dev);
			bwin->midicontrolval[cc][i] = value;
//				= ((brightonDevice *) p->devlocn[ind].dev)->value;
			break;
		}
	}

	if (i == BRIGHTON_GANG_COUNT)
	{
		bwin->midicontrol[cc][(i = BRIGHTON_GANG_COUNT - 1)]
			= ((struct brightonDevice *) p->devlocn[ind].dev);
		bwin->midicontrolval[cc][i] = value;
	}

	/*
	 * Build our scaling value
	 */
	for (i = 0; i < BRIGHTON_GANG_COUNT; i++)
	{
		if (bwin->midicontrolval[cc][i] > maxvalue)
			maxvalue = bwin->midicontrolval[cc][i];
	}

	if (maxvalue == 0.0)
		bwin->midicontrolscaler[cc] = 1.0;
	else
		bwin->midicontrolscaler[cc] = 1.0 / maxvalue;

	for (i = 0; i < BRIGHTON_GANG_COUNT; i++)
		if (bwin->midicontrol[cc][i] != NULL)
			printf("MIDI CC %i->%i/%i@%x (index %i: %f scaled by %f)\n",
				cc,
				((brightonDevice *) bwin->midicontrol[cc][i])->panel,
				((brightonDevice *) bwin->midicontrol[cc][i])->index,
				bwin->midicontrol[cc][i],
				i,
				bwin->midicontrolval[cc][i],
				bwin->midicontrolscaler[cc]);
}

/*
 * Parse configuration file. Read the panel and index numbers for the devices
 * and find the actual device configure code for the controllers.
 */
brightonReadConfiguration(brightonWindow *bwin, brightonApp *app,
int channel, char *synth)
{
	FILE *fd;
	char path[256];
	char param[256];
	float value;

	printf("Read Configuration: %s\n", synth);

	sprintf(path, "%s/memory/profiles/%s", global.home, synth);
	printf("%s\n", path);

	if ((fd = fopen(path, "r")) == NULL)
		return;

	while (fgets(param, 256, fd) != NULL)
	{
		if (param[0] == '#')
			continue;

//		printf("read line %s", param);

		if (strncmp(param, "CC", 2) == 0)
			brightonSetControl(bwin, app, channel, &param[4]);

		if (strncmp(param, "CM", 2) == 0)
			brightonMapControl(bwin, app, channel, &param[4]);
	}

	fclose(fd);
}

/*
 * Save a configuration file. This should take the panel and index numbers for
 * each of the devices in the table, for a specific MIDI channel, and write 
 * them out to a file with the controller ID.
 */
brightonWriteConfiguration(brightonWindow *bwin, char *synth, int channel)
{
	int j, i, fd, map;
	char path[256];
	char param[256];

	printf("Write CC Configuration file not implemented: %s\n", synth);

	sprintf(path, "%s/memory/profiles/%s", global.home, synth);
	printf("%s\n", path);

	if ((fd = open(path, O_WRONLY|O_TRUNC|O_CREAT, 0644)) < 0)
		return;

	/*
	 * This could be cleaned up a bit
	 */
	write(fd, pheader, strlen(pheader));

	for (i = 0; i < 128; i++)
	{
		if (bwin->midimap[i] == i)
			continue;

		sprintf(param, "CM: %i	   %i\n", i, bwin->midimap[i]);
		printf("%s", param);
		write(fd, param, strlen(param));
	}

	for (i = 0; i < 128; i++)
	{
		for (j = 0; j < BRIGHTON_GANG_COUNT; j++)
		{
			if (bwin->midicontrol[i][j] != 0)
			{
				sprintf(param, "CC: %i	   %i/%i %i\n",
					i,
					((brightonDevice *) bwin->midicontrol[i][j])->panel,
					((brightonDevice *) bwin->midicontrol[i][j])->index,
					(int) (bwin->midicontrolval[i][j] * 16384));
				printf("%s", param);
				write(fd, param, strlen(param));
			}
		}
	}

	close(fd);
}

/*
 * Common dispatch routine for controller to GUI events. Needs to find the
 * right synth, not trivial - may have multiple on this channel, then needs to
 * call parsing and mapping for each.
 */
brightonMidiInput(bristolMidiMsg *msg, guimain *global)
{
	int i, j;
	float maxvalue = 0.0;

//printf("Midi msg: %x, %i %i, %i\n", msg->command, msg->channel,
//global->synths->midichannel);

#warning that we need to scan the synths to match the MIDI channel
	/*
	 * We should consider what to do with channel changes, but we are not
	 * responsible for those. This is what the controller sent although we
	 * could copy the values across from one table to the other?
	 *
	 * We should also consider an interface to allow for note on/off events,
	 * it would need to register with panel ID and transpose.
	 * We should also consider an interface to allow for pitch and mod wheel
	 * registration.
	 */

	if (msg->command != MIDI_CONTROL)
		return;

	bristolMidiToGM2(global->synths->win, msg);

	if ((msg->GM2.c_id < 0) || (msg->GM2.c_id > 127))
		return;

	if (msg->channel != global->synths->midichannel)
		return;

//printf("Midi Ctrl: %i\n", msg->GM2.c_id);

	if (confflag > 0) {
		/*
		 * Link this CC to the given device, by MIDI channel? This also
		 * needs to be made into a GM-2 controller (perhaps that should
		 * be buried in the MIDI MSG structure - native and GM-2?
		 */
		for (j = 0; j < BRIGHTON_GANG_COUNT; j++)
		{
			if (confdev[j] == 0)
				continue;

			for (i = 0; i < BRIGHTON_GANG_COUNT; i++)
			{
				if ((brightonDevice *)
					global->synths->win->midicontrol[msg->GM2.c_id][i]
						== confdev[j])
				{
					printf("Controller Registration Cleared: %i -> %i/%i@%x\n",
						msg->GM2.c_id, confdev[j]->panel,
						confdev[j]->index, confdev[j]);
					global->synths->win->midicontrol[msg->GM2.c_id][i] = 0;
					break;
				} else {
					if (global->synths->win->midicontrol[msg->GM2.c_id][i] != 0)
					{
						if (global->synths->win->midicontrolval[msg->GM2.c_id][i]
							> maxvalue)
							maxvalue = global->synths->win->midicontrolval[msg->GM2.c_id][i];

						continue;
					}

					global->synths->win->midicontrol[msg->GM2.c_id][i]
						= (struct brightonDevice *) confdev[j];
					global->synths->win->midicontrolval[msg->GM2.c_id][i]
						= confval[j];
//						= ((brightonDevice *) confdev[j])->value;
					printf("Controller Registration Honoured: %i -> %i/%i@%x %f\n",
						msg->GM2.c_id, confdev[j]->panel,
						confdev[j]->index, confdev[j], confval[j]);
//					if (global->synths->win->midicontrolval[msg->GM2.c_id][i]
//						> maxvalue)
//						maxvalue = global->synths->win->midicontrolval[msg->GM2.c_id][i];
					break;
				}
			}
		}
		confflag = 0;
		for (i = 0; i < BRIGHTON_GANG_COUNT; i++)
		{
			if (global->synths->win->midicontrolval[msg->GM2.c_id][i]
				> maxvalue)
				maxvalue = global->synths->win->midicontrolval[msg->GM2.c_id][i];
			confdev[i] = 0;
		}
		/*
		 * We now have a max value, we need to work on a scaler such that 
		 * this gang will scale to max
		 */
		if (maxvalue == 0.0f)
		{
			global->synths->win->midicontrolscaler[msg->GM2.c_id] = 1.0;
		} else {
			global->synths->win->midicontrolscaler[msg->GM2.c_id] =
				1.0 / maxvalue;
		}
	} else {
		brightonEvent event;

		/*
		 * This next routine should use a parser on the message that will
		 * build a GM-2 value into the msg, this will take care of things like
		 * controllers that have fine and coarse resolution and will suitably
		 * adjust the value to a suitable float, also changing the controller
		 * number if this is the fine adjustment of a coarse control. This 
		 * may already have been done by the library?
		 */
		for (i = 0; i < BRIGHTON_GANG_COUNT; i++)
		{
			if (global->synths->win->midicontrol[msg->GM2.c_id][i] != 0)
			{
				brightonIResource *p;
				int panel, cc, ind, channel;

				panel = ((brightonDevice *)
					global->synths->win->midicontrol[msg->GM2.c_id][i])->panel;
				ind = ((brightonDevice *)
					global->synths->win->midicontrol[msg->GM2.c_id][i])->index;
				cc = msg->GM2.c_id;
				channel = msg->channel;

				/*
				 * This value needs to be scaled according to the limits that
				 * were set by the original registration. We should check for
				 * max values.
				 */
//				event.value = msg->GM2.value;
				if ((event.value = msg->GM2.value *
					global->synths->win->midicontrolval[msg->GM2.c_id][i] *
					global->synths->win->midicontrolscaler[msg->GM2.c_id])
					> 1.0)
					event.value = 1.0;
				event.command = event.type = BRIGHTON_PARAMCHANGE;

				p = (brightonIResource *)
					&global->synths->win->app->resources[panel];
				if (p->devlocn[ind].to != 1.0)
					event.value *= p->devlocn[ind].to;

				/*
				if (((brightonDevice *)
					global->synths->win->midicontrol[msg->GM2.c_id][i])->device
					== 1)
					event.value = 1.0 - event.value;
				 */

//			printf(
//				"global->synths->win->midicontrol[%i][%i]->%x(%x, &event): %f\n",
//				msg->GM2.c_id,
//				msg->channel,
//				global->synths->win->midicontrol[msg->GM2.c_id]->configure,
//				global->synths->win->midicontrol[msg->GM2.c_id],
//				event.value);

				((brightonDevice *)
					global->synths->win->midicontrol[msg->GM2.c_id][i])->configure(
					global->synths->win->midicontrol[msg->GM2.c_id][i], &event);
			}
		}
	}
}

/*
 * Common dispatch routine for Event selection to controller events.
 */
brightonRegisterController(brightonDevice *dev)
{
	int i;

	for (i = 0; i < BRIGHTON_GANG_COUNT; i++)
	{
		if (confdev[i] == dev)
		{
			printf("Controller Registration %i Cleared: ? -> %i/%i@%08x\n",
				i, dev->panel, dev->index, dev);

			confdev[i] = 0;
			if (--confflag <= 0)
			{
				confflag = 0;
				for (i = 0; i < BRIGHTON_GANG_COUNT; i++)
					confdev[i] = 0;
			}

			return;
		}
	}

	/*
	 * This should go into a table. Then, when we get the target controller
	 * we change the device value and consider a paramchange message to be
	 * sent to update the GUI (and send the value?).
	confflag = 0;
	confdev = dev;
	 */
	for (i = 0; i < BRIGHTON_GANG_COUNT; i++)
	{
		if (confdev[i] == 0)
		{
			confdev[i] = dev;

			if (dev->device == 1)
				confval[i] = 1.0 - dev->value;
			else
				confval[i] = dev->value;

			if (confval[i] == 0.0)
				confval[i] = 1.0;

			break;
		}
	}
	if (++confflag >= BRIGHTON_GANG_COUNT)
	{
		confflag = BRIGHTON_GANG_COUNT;
		confdev[BRIGHTON_GANG_COUNT - 1] = dev;
	}

	printf("Controller Registration %i Request: ? -> %i/%i@%08x %f\n",
		i, dev->panel, dev->index, dev, dev->value);
}

