
/*
 *  Diverse Bristol audio routines.
 *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2006
 *
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

/*
 * These need to be redone. Each white key should be placed first, then the
 * black keys layered on top. That way we can get better graphics with shading
 * on the white keys for each black key.
 */

#ifndef BRIGHTONKEYS_H
#define BRIGHTONKEYS_H

extern guimain global;

#define KEY_COUNT 61
#define KEY_COUNT_2OCTAVE 29
#define KEY_COUNT_3OCTAVE 44
#define VKEY_COUNT 48
#define KEY_COUNT_5OCTAVE KEY_COUNT
#define KEY_COUNT_6_OCTAVE 73

/*
 * This whole definition should be moved into a separate routine, and called
 * for a given keyboard size - 37 note (three oct), 61 note (5 oct) or a full
 * 88 keys. Should also consider 73 note for a Rhodes....
 */
#define KW (1000 / 36)
#define FKW (995 - 35 * KW)
#define KFBW ((KW * 3) / 2)
#define KFWD ((KW * 3) / 4)

#define KMW 20
#define KIW 28
#define KIH 666
#define KIL 334
#define KIS 0

static brightonLocations keys[KEY_COUNT_5OCTAVE] = {
	{"", 2, 0 * KW, 0, KFWD, 1000, 0, 1, 0,
		"bitmaps/newkeys/WhiteFUp.xpm", "bitmaps/newkeys/WhiteFDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 0 * KW + KW/2, 0, KW, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 1 * KW, KIH, KW, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 1 * KW + KW/2, 0, KW, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 2 * KW, KIH, KW, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 3 * KW, 0, KFWD, 1000, 0, 1, 0,
		"bitmaps/newkeys/WhiteFUp.xpm", "bitmaps/newkeys/WhiteFDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 3 * KW + KW/2, 0, KW, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 4 * KW, KIH, KW, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 4 * KW + KW/2, 0, KW, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 5 * KW, KIH, KW, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 5 * KW + KW/2, 0, KW, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 6 * KW, KIH, KW, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},

	{"", 2, 7 * KW, 0, KFWD, 1000, 0, 1, 0,
		"bitmaps/newkeys/WhiteFUp.xpm", "bitmaps/newkeys/WhiteFDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 7 * KW + KW/2, 0, KW, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 8 * KW, KIH, KW, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 8 * KW + KW/2, 0, KW, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 9 * KW, KIH, KW, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 10 * KW, 0, KFWD, 1000, 0, 1, 0,
		"bitmaps/newkeys/WhiteFUp.xpm", "bitmaps/newkeys/WhiteFDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 10 * KW + KW/2, 0, KW, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 11 * KW, KIH, KW, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 11 * KW + KW/2, 0, KW, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 12 * KW, KIH, KW, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 12 * KW + KW/2, 0, KW, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 13 * KW, KIH, KW, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},

	{"", 2, 14 * KW, 0, KFWD, 1000, 0, 1, 0,
		"bitmaps/newkeys/WhiteFUp.xpm", "bitmaps/newkeys/WhiteFDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 14 * KW + KW/2, 0, KW, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 15 * KW, KIH, KW, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 15 * KW + KW/2, 0, KW, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 16 * KW, KIH, KW, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 17 * KW, 0, KFWD, 1000, 0, 1, 0,
		"bitmaps/newkeys/WhiteFUp.xpm", "bitmaps/newkeys/WhiteFDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 17 * KW + KW/2, 0, KW, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 18 * KW, KIH, KW, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 18 * KW + KW/2, 0, KW, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 19 * KW, KIH, KW, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 19 * KW + KW/2, 0, KW, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 20 * KW, KIH, KW, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},

	{"", 2, 21 * KW, 0, KFWD, 1000, 0, 1, 0,
		"bitmaps/newkeys/WhiteFUp.xpm", "bitmaps/newkeys/WhiteFDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 21 * KW + KW/2, 0, KW, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 22 * KW, KIH, KW, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 22 * KW + KW/2, 0, KW, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 23 * KW, KIH, KW, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 24 * KW, 0, KFWD, 1000, 0, 1, 0,
		"bitmaps/newkeys/WhiteFUp.xpm", "bitmaps/newkeys/WhiteFDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 24 * KW + KW/2, 0, KW, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 25 * KW, KIH, KW, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 25 * KW + KW/2, 0, KW, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 26 * KW, KIH, KW, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 26 * KW + KW/2, 0, KW, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 27 * KW, KIH, KW, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},

	{"", 2, 28 * KW, 0, KFWD, 1000, 0, 1, 0,
		"bitmaps/newkeys/WhiteFUp.xpm", "bitmaps/newkeys/WhiteFDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 28 * KW + KW/2, 0, KW, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 29 * KW, KIH, KW, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 29 * KW + KW/2, 0, KW, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 30 * KW, KIH, KW, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 31 * KW, 0, KFWD, 1000, 0, 1, 0,
		"bitmaps/newkeys/WhiteFUp.xpm", "bitmaps/newkeys/WhiteFDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 31 * KW + KW/2, 0, KW, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 32 * KW, KIH, KW, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 32 * KW + KW/2, 0, KW, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 33 * KW, KIH, KW, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 33 * KW + KW/2, 0, KW, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 34 * KW, KIH, KW, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},

	{"", 2, 35 * KW, 0, KW, 1000, 0, 1, 0,
		"bitmaps/newkeys/WhiteLUp.xpm", "bitmaps/newkeys/WhiteLDown.xpm", BRIGHTON_NOSHADOW},
};

#define K6W 1000 / 43
#define F6KW 1000 - 43 * KW

#define K6FW (K6W * 3 / 4)

#define K6MW 13
#define K6IW 22
#define K6IH 666
#define K6IL 334
#define K6IS 20

static brightonLocations keys6octave[KEY_COUNT_6_OCTAVE] = {
	{"", 2, 0 * K6W, 0, K6FW, 1000, 0, 1, 0,
		"bitmaps/newkeys/WhiteDFUp.xpm", "bitmaps/newkeys/WhiteDFDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 0 * K6W + K6W/2, 0, K6W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 1 * K6W, KIH, K6W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 1 * K6W + K6W/2, 0, K6W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 2 * K6W, KIH, K6W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 3 * K6W, 0, K6FW, 1000, 0, 1, 0,
		"bitmaps/newkeys/WhiteDFUp.xpm", "bitmaps/newkeys/WhiteDFDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 3 * K6W + K6W/2, 0, K6W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 4 * K6W, KIH, K6W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 4 * K6W + K6W/2, 0, K6W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 5 * K6W, KIH, K6W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 5 * K6W + K6W/2, 0, K6W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 6 * K6W, KIH, K6W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},

	{"", 2, 7 * K6W, 0, K6FW, 1000, 0, 1, 0,
		"bitmaps/newkeys/WhiteDFUp.xpm", "bitmaps/newkeys/WhiteDFDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 7 * K6W + K6W/2, 0, K6W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 8 * K6W, KIH, K6W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 8 * K6W + K6W/2, 0, K6W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 9 * K6W, KIH, K6W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 10 * K6W, 0, K6FW, 1000, 0, 1, 0,
		"bitmaps/newkeys/WhiteDFUp.xpm", "bitmaps/newkeys/WhiteDFDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 10 * K6W + K6W/2, 0, K6W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 11 * K6W, KIH, K6W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 11 * K6W + K6W/2, 0, K6W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 12 * K6W, KIH, K6W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 12 * K6W + K6W/2, 0, K6W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 13 * K6W, KIH, K6W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},

	{"", 2, 14 * K6W, 0, K6FW, 1000, 0, 1, 0,
		"bitmaps/newkeys/WhiteDFUp.xpm", "bitmaps/newkeys/WhiteDFDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 14 * K6W + K6W/2, 0, K6W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 15 * K6W, KIH, K6W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 15 * K6W + K6W/2, 0, K6W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 16 * K6W, KIH, K6W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 17 * K6W, 0, K6FW, 1000, 0, 1, 0,
		"bitmaps/newkeys/WhiteDFUp.xpm", "bitmaps/newkeys/WhiteDFDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 17 * K6W + K6W/2, 0, K6W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 18 * K6W, KIH, K6W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 18 * K6W + K6W/2, 0, K6W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 19 * K6W, KIH, K6W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 19 * K6W + K6W/2, 0, K6W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 20 * K6W, KIH, K6W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},

	{"", 2, 21 * K6W, 0, K6FW, 1000, 0, 1, 0,
		"bitmaps/newkeys/WhiteDFUp.xpm", "bitmaps/newkeys/WhiteDFDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 21 * K6W + K6W/2, 0, K6W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 22 * K6W, KIH, K6W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 22 * K6W + K6W/2, 0, K6W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 23 * K6W, KIH, K6W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 24 * K6W, 0, K6FW, 1000, 0, 1, 0,
		"bitmaps/newkeys/WhiteDFUp.xpm", "bitmaps/newkeys/WhiteDFDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 24 * K6W + K6W/2, 0, K6W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 25 * K6W, KIH, K6W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 25 * K6W + K6W/2, 0, K6W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 26 * K6W, KIH, K6W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 26 * K6W + K6W/2, 0, K6W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 27 * K6W, KIH, K6W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},

	{"", 2, 28 * K6W, 0, K6FW, 1000, 0, 1, 0,
		"bitmaps/newkeys/WhiteDFUp.xpm", "bitmaps/newkeys/WhiteDFDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 28 * K6W + K6W/2, 0, K6W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 29 * K6W, KIH, K6W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 29 * K6W + K6W/2, 0, K6W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 30 * K6W, KIH, K6W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 31 * K6W, 0, K6FW, 1000, 0, 1, 0,
		"bitmaps/newkeys/WhiteDFUp.xpm", "bitmaps/newkeys/WhiteDFDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 31 * K6W + K6W/2, 0, K6W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 32 * K6W, KIH, K6W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 32 * K6W + K6W/2, 0, K6W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 33 * K6W, KIH, K6W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 33 * K6W + K6W/2, 0, K6W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 34 * K6W, KIH, K6W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},

	{"", 2, 35 * K6W, 0, K6FW, 1000, 0, 1, 0,
		"bitmaps/newkeys/WhiteDFUp.xpm", "bitmaps/newkeys/WhiteDFDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 35 * K6W + K6W/2, 0, K6W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 36 * K6W, KIH, K6W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 36 * K6W + K6W/2, 0, K6W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 37 * K6W, KIH, K6W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 38 * K6W, 0, K6FW, 1000, 0, 1, 0,
		"bitmaps/newkeys/WhiteDFUp.xpm", "bitmaps/newkeys/WhiteDFDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 38 * K6W + K6W/2, 0, K6W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 39 * K6W, KIH, K6W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 39 * K6W + K6W/2, 0, K6W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 40 * K6W, KIH, K6W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 40 * K6W + K6W/2, 0, K6W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 41 * K6W, KIH, K6W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},

	{"", 2, 42 * K6W, 0, K6W, 1000, 0, 1, 0,
		"bitmaps/newkeys/WhiteLUp.xpm", "bitmaps/newkeys/WhiteLDown.xpm", BRIGHTON_NOSHADOW},
};

static brightonLocations keys6hammond[KEY_COUNT_6_OCTAVE] = {
	{"", 2, 0 * K6W, 0, K6FW, 1000, 0, 1, 0,
		"bitmaps/newkeys/WhiteFRUp.xpm", "bitmaps/newkeys/WhiteFRDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 0 * K6W + K6W/2, 0, K6W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackRUp.xpm", "bitmaps/newkeys/BlackRDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 1 * K6W, KIH, K6W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteRUp.xpm", "bitmaps/newkeys/WhiteRDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 1 * K6W + K6W/2, 0, K6W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackRUp.xpm", "bitmaps/newkeys/BlackRDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 2 * K6W, KIH, K6W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteRUp.xpm", "bitmaps/newkeys/WhiteRDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 3 * K6W, 0, K6FW, 1000, 0, 1, 0,
		"bitmaps/newkeys/WhiteFRUp.xpm", "bitmaps/newkeys/WhiteFRDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 3 * K6W + K6W/2, 0, K6W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackRUp.xpm", "bitmaps/newkeys/BlackRDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 4 * K6W, KIH, K6W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteRUp.xpm", "bitmaps/newkeys/WhiteRDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 4 * K6W + K6W/2, 0, K6W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackRUp.xpm", "bitmaps/newkeys/BlackRDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 5 * K6W, KIH, K6W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteRUp.xpm", "bitmaps/newkeys/WhiteRDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 5 * K6W + K6W/2, 0, K6W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackRUp.xpm", "bitmaps/newkeys/BlackRDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 6 * K6W, KIH, K6W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteRUp.xpm", "bitmaps/newkeys/WhiteRDown.xpm", BRIGHTON_NOSHADOW},

	{"", 2, 7 * K6W, 0, K6FW, 1000, 0, 1, 0,
		"bitmaps/newkeys/WhiteHFUp.xpm", "bitmaps/newkeys/WhiteHFDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 7 * K6W + K6W/2, 0, K6W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 8 * K6W, KIH, K6W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 8 * K6W + K6W/2, 0, K6W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 9 * K6W, KIH, K6W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 10 * K6W, 0, K6FW, 1000, 0, 1, 0,
		"bitmaps/newkeys/WhiteHFUp.xpm", "bitmaps/newkeys/WhiteHFDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 10 * K6W + K6W/2, 0, K6W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 11 * K6W, KIH, K6W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 11 * K6W + K6W/2, 0, K6W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 12 * K6W, KIH, K6W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 12 * K6W + K6W/2, 0, K6W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 13 * K6W, KIH, K6W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},

	{"", 2, 14 * K6W, 0, K6FW, 1000, 0, 1, 0,
		"bitmaps/newkeys/WhiteHFUp.xpm", "bitmaps/newkeys/WhiteHFDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 14 * K6W + K6W/2, 0, K6W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 15 * K6W, KIH, K6W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 15 * K6W + K6W/2, 0, K6W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 16 * K6W, KIH, K6W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 17 * K6W, 0, K6FW, 1000, 0, 1, 0,
		"bitmaps/newkeys/WhiteHFUp.xpm", "bitmaps/newkeys/WhiteHFDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 17 * K6W + K6W/2, 0, K6W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 18 * K6W, KIH, K6W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 18 * K6W + K6W/2, 0, K6W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 19 * K6W, KIH, K6W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 19 * K6W + K6W/2, 0, K6W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 20 * K6W, KIH, K6W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},

	{"", 2, 21 * K6W, 0, K6FW, 1000, 0, 1, 0,
		"bitmaps/newkeys/WhiteHFUp.xpm", "bitmaps/newkeys/WhiteHFDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 21 * K6W + K6W/2, 0, K6W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 22 * K6W, KIH, K6W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 22 * K6W + K6W/2, 0, K6W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 23 * K6W, KIH, K6W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 24 * K6W, 0, K6FW, 1000, 0, 1, 0,
		"bitmaps/newkeys/WhiteHFUp.xpm", "bitmaps/newkeys/WhiteHFDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 24 * K6W + K6W/2, 0, K6W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 25 * K6W, KIH, K6W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 25 * K6W + K6W/2, 0, K6W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 26 * K6W, KIH, K6W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 26 * K6W + K6W/2, 0, K6W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 27 * K6W, KIH, K6W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},

	{"", 2, 28 * K6W, 0, K6FW, 1000, 0, 1, 0,
		"bitmaps/newkeys/WhiteHFUp.xpm", "bitmaps/newkeys/WhiteHFDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 28 * K6W + K6W/2, 0, K6W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 29 * K6W, KIH, K6W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 29 * K6W + K6W/2, 0, K6W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 30 * K6W, KIH, K6W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 31 * K6W, 0, K6FW, 1000, 0, 1, 0,
		"bitmaps/newkeys/WhiteHFUp.xpm", "bitmaps/newkeys/WhiteHFDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 31 * K6W + K6W/2, 0, K6W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 32 * K6W, KIH, K6W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 32 * K6W + K6W/2, 0, K6W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 33 * K6W, KIH, K6W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 33 * K6W + K6W/2, 0, K6W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 34 * K6W, KIH, K6W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},

	{"", 2, 35 * K6W, 0, K6FW, 1000, 0, 1, 0,
		"bitmaps/newkeys/WhiteHFUp.xpm", "bitmaps/newkeys/WhiteHFDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 35 * K6W + K6W/2, 0, K6W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 36 * K6W, KIH, K6W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 36 * K6W + K6W/2, 0, K6W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 37 * K6W, KIH, K6W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 38 * K6W, 0, K6FW, 1000, 0, 1, 0,
		"bitmaps/newkeys/WhiteHFUp.xpm", "bitmaps/newkeys/WhiteHFDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 38 * K6W + K6W/2, 0, K6W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 39 * K6W, KIH, K6W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 39 * K6W + K6W/2, 0, K6W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 40 * K6W, KIH, K6W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 40 * K6W + K6W/2, 0, K6W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 41 * K6W, KIH, K6W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},

	{"", 2, 42 * K6W, 0, K6W, 1000, 0, 1, 0,
		"bitmaps/newkeys/WhiteLUp.xpm", "bitmaps/newkeys/WhiteLDown.xpm", BRIGHTON_NOSHADOW},
};

#define K2W (1000 / 17)
#define K2FW (K2W * 3 / 4)

#define K2MW 18
#define K2IW 40
#define K2IH 600
#define K2IL 410
#define K2IS 33

static brightonLocations keys2octave[KEY_COUNT_2OCTAVE] = {
	{"", 2, 0 * K2W, 0, K2FW, 1000, 0, 1, 0,
		"bitmaps/newkeys/WhiteFUp.xpm", "bitmaps/newkeys/WhiteFDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 0 * K2W + K2W/2, 0, K2W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 1 * K2W, KIH, K2W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 1 * K2W + K2W/2, 0, K2W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 2 * K2W, KIH, K2W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 3 * K2W, 0, K2FW, 1000, 0, 1, 0,
		"bitmaps/newkeys/WhiteFUp.xpm", "bitmaps/newkeys/WhiteFDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 3 * K2W + K2W/2, 0, K2W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 4 * K2W, KIH, K2W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 4 * K2W + K2W/2, 0, K2W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 5 * K2W, KIH, K2W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 5 * K2W + K2W/2, 0, K2W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 6 * K2W, KIH, K2W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},

	{"", 2, 7 * K2W, 0, K2FW, 1000, 0, 1, 0,
		"bitmaps/newkeys/WhiteFUp.xpm", "bitmaps/newkeys/WhiteFDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 7 * K2W + K2W/2, 0, K2W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 8 * K2W, KIH, K2W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 8 * K2W + K2W/2, 0, K2W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 9 * K2W, KIH, K2W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 10 * K2W, 0, K2FW, 1000, 0, 1, 0,
		"bitmaps/newkeys/WhiteFUp.xpm", "bitmaps/newkeys/WhiteFDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 10 * K2W + K2W/2, 0, K2W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 11 * K2W, KIH, K2W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 11 * K2W + K2W/2, 0, K2W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 12 * K2W, KIH, K2W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 12 * K2W + K2W/2, 0, K2W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 13 * K2W, KIH, K2W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},

	{"", 2, 14 * K2W, 0, K2FW, 1000, 0, 1, 0,
		"bitmaps/newkeys/WhiteFUp.xpm", "bitmaps/newkeys/WhiteFDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 14 * K2W + K2W/2, 0, K2W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 15 * K2W, KIH, K2W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 15 * K2W + K2W/2, 0, K2W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 16 * K2W, KIH, K2W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
//	{"", 2, 17 * K2W, 0, K2FW, 1000, 0, 1, 0,
//		"bitmaps/newkeys/WhiteFUp.xpm", "bitmaps/newkeys/WhiteFDown.xpm", BRIGHTON_NOSHADOW},
};

#define K3W (1000 / 26)
#define K3FW (K3W * 3 / 4)

#define K3MW 18
#define K3IW 40
#define K3IH 600
#define K3IL 410
#define K3IS 33

static brightonLocations keys3octave[KEY_COUNT_3OCTAVE] = {
	{"", 2, 0 * K3W, 0, K3FW, 1000, 0, 1, 0,
		"bitmaps/newkeys/WhiteFUp.xpm", "bitmaps/newkeys/WhiteFDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 0 * K3W + K3W/2, 0, K3W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 1 * K3W, KIH, K3W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 1 * K3W + K3W/2, 0, K3W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 2 * K3W, KIH, K3W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 3 * K3W, 0, K3FW, 1000, 0, 1, 0,
		"bitmaps/newkeys/WhiteFUp.xpm", "bitmaps/newkeys/WhiteFDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 3 * K3W + K3W/2, 0, K3W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 4 * K3W, KIH, K3W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 4 * K3W + K3W/2, 0, K3W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 5 * K3W, KIH, K3W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 5 * K3W + K3W/2, 0, K3W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 6 * K3W, KIH, K3W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},

	{"", 2, 7 * K3W, 0, K3FW, 1000, 0, 1, 0,
		"bitmaps/newkeys/WhiteFUp.xpm", "bitmaps/newkeys/WhiteFDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 7 * K3W + K3W/2, 0, K3W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 8 * K3W, KIH, K3W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 8 * K3W + K3W/2, 0, K3W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 9 * K3W, KIH, K3W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 10 * K3W, 0, K3FW, 1000, 0, 1, 0,
		"bitmaps/newkeys/WhiteFUp.xpm", "bitmaps/newkeys/WhiteFDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 10 * K3W + K3W/2, 0, K3W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 11 * K3W, KIH, K3W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 11 * K3W + K3W/2, 0, K3W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 12 * K3W, KIH, K3W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 12 * K3W + K3W/2, 0, K3W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 13 * K3W, KIH, K3W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},

	{"", 2, 14 * K3W, 0, K3FW, 1000, 0, 1, 0,
		"bitmaps/newkeys/WhiteFUp.xpm", "bitmaps/newkeys/WhiteFDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 14 * K3W + K3W/2, 0, K3W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 15 * K3W, KIH, K3W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 15 * K3W + K3W/2, 0, K3W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 16 * K3W, KIH, K3W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 17 * K3W, 0, K3FW, 1000, 0, 1, 0,
		"bitmaps/newkeys/WhiteFUp.xpm", "bitmaps/newkeys/WhiteFDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 17 * K3W + K3W/2, 0, K3W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 18 * K3W, KIH, K3W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 18 * K3W + K3W/2, 0, K3W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 19 * K3W, KIH, K3W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 19 * K3W + K3W/2, 0, K3W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 20 * K3W, KIH, K3W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},

	{"", 2, 21 * K3W, 0, K3FW, 1000, 0, 1, 0,
		"bitmaps/newkeys/WhiteFUp.xpm", "bitmaps/newkeys/WhiteFDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 21 * K3W + K3W/2, 0, K3W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 22 * K3W, KIH, K3W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 22 * K3W + K3W/2, 0, K3W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 23 * K3W, KIH, K3W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 24 * K3W, 0, K3FW, 1000, 0, 1, 0,
		"bitmaps/newkeys/WhiteFUp.xpm", "bitmaps/newkeys/WhiteFDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 24 * K3W + K3W/2, 0, K3W, KIH, 0, 1, 0,
		"bitmaps/newkeys/BlackUp.xpm", "bitmaps/newkeys/BlackDown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 25 * K3W, KIH, K3W, KIL, 0, 1, 0,
		"bitmaps/newkeys/WhiteUp.xpm", "bitmaps/newkeys/WhiteDown.xpm", BRIGHTON_NOSHADOW},
};

#define KVW 1000 / 28

#define KVMW 18
#define KVIW 40
#define KVIH 600
#define KVIL 410
#define KVIS 33

static brightonLocations vkeys[VKEY_COUNT] = {
	{"", 2, 0 * KVW, KVIH, KVW, KVIL, 0, 1, 0,
		"bitmaps/keys/wrevup.xpm", "bitmaps/keys/wrevdown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 0 * KVW, 0, KVIW + 7, KVIH, 0, 1, 0,
		"bitmaps/keys/brevup.xpm", "bitmaps/keys/brevdown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 1 * KVW, KVIH, KVW, KVIL, 0, 1, 0,
		"bitmaps/keys/wrevup.xpm", "bitmaps/keys/wrevdown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 1 * KVW + KVIS, 0, KVMW, KVIH, 0, 1, 0,
		"bitmaps/keys/brevmup.xpm", "bitmaps/keys/brevmdown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 2 * KVW, KVIH, KVW, KVIL, 0, 1, 0,
		"bitmaps/keys/wrevup.xpm", "bitmaps/keys/wrevdown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 3 * KVW, KVIH, KVW, KVIL, 0, 1, 0,
		"bitmaps/keys/wrevup.xpm", "bitmaps/keys/wrevdown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 3 * KVW, 0, KVIW + 10, KVIH, 0, 1, 0,
		"bitmaps/keys/brevup.xpm", "bitmaps/keys/brevdown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 4 * KVW, KVIH, KVW, KVIL, 0, 1, 0,
		"bitmaps/keys/wrevup.xpm", "bitmaps/keys/wrevdown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 4 * KVW + KVIS, 0, KVMW, KVIH, 0, 1, 0,
		"bitmaps/keys/brevmup.xpm", "bitmaps/keys/brevmdown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 5 * KVW, KVIH, KVW, KVIL, 0, 1, 0,
		"bitmaps/keys/wrevup.xpm", "bitmaps/keys/wrevdown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 5 * KVW + KVIS, 0, KVMW, KVIH, 0, 1, 0,
		"bitmaps/keys/brevmup.xpm", "bitmaps/keys/brevmdown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 6 * KVW, KVIH, KVW, KVIL, 0, 1, 0,
		"bitmaps/keys/wrevup.xpm", "bitmaps/keys/wrevdown.xpm", BRIGHTON_NOSHADOW},

	{"", 2, 7 * KVW, KVIH, KVW, KVIL, 0, 1, 0,
		"bitmaps/keys/wrevup.xpm", "bitmaps/keys/wrevdown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 7 * KVW, 0, KVIW + 7, KVIH, 0, 1, 0,
		"bitmaps/keys/brevup.xpm", "bitmaps/keys/brevdown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 8 * KVW, KVIH, KVW, KVIL, 0, 1, 0,
		"bitmaps/keys/wrevup.xpm", "bitmaps/keys/wrevdown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 8 * KVW + KVIS, 0, KVMW, KVIH, 0, 1, 0,
		"bitmaps/keys/brevmup.xpm", "bitmaps/keys/brevmdown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 9 * KVW, KVIH, KVW, KVIL, 0, 1, 0,
		"bitmaps/keys/wrevup.xpm", "bitmaps/keys/wrevdown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 10 * KVW, KVIH, KVW, KVIL, 0, 1, 0,
		"bitmaps/keys/wrevup.xpm", "bitmaps/keys/wrevdown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 10 * KVW, 0, KVIW + 10, KVIH, 0, 1, 0,
		"bitmaps/keys/brevup.xpm", "bitmaps/keys/brevdown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 11 * KVW, KVIH, KVW, KVIL, 0, 1, 0,
		"bitmaps/keys/wrevup.xpm", "bitmaps/keys/wrevdown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 11 * KVW + KVIS, 0, KVMW, KVIH, 0, 1, 0,
		"bitmaps/keys/brevmup.xpm", "bitmaps/keys/brevmdown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 12 * KVW, KVIH, KVW, KVIL, 0, 1, 0,
		"bitmaps/keys/wrevup.xpm", "bitmaps/keys/wrevdown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 12 * KVW + KVIS, 0, KVMW, KVIH, 0, 1, 0,
		"bitmaps/keys/brevmup.xpm", "bitmaps/keys/brevmdown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 13 * KVW, KVIH, KVW, KVIL, 0, 1, 0,
		"bitmaps/keys/wrevup.xpm", "bitmaps/keys/wrevdown.xpm", BRIGHTON_NOSHADOW},

	{"", 2, 14 * KVW, KVIH, KVW, KVIL, 0, 1, 0,
		"bitmaps/keys/wrevup.xpm", "bitmaps/keys/wrevdown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 14 * KVW, 0, KVIW + 7, KVIH, 0, 1, 0,
		"bitmaps/keys/brevup.xpm", "bitmaps/keys/brevdown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 15 * KVW, KVIH, KVW, KVIL, 0, 1, 0,
		"bitmaps/keys/wrevup.xpm", "bitmaps/keys/wrevdown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 15 * KVW + KVIS, 0, KVMW, KVIH, 0, 1, 0,
		"bitmaps/keys/brevmup.xpm", "bitmaps/keys/brevmdown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 16 * KVW, KVIH, KVW, KVIL, 0, 1, 0,
		"bitmaps/keys/wrevup.xpm", "bitmaps/keys/wrevdown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 17 * KVW, KVIH, KVW, KVIL, 0, 1, 0,
		"bitmaps/keys/wrevup.xpm", "bitmaps/keys/wrevdown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 17 * KVW, 0, KVIW + 10, KVIH, 0, 1, 0,
		"bitmaps/keys/brevup.xpm", "bitmaps/keys/brevdown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 18 * KVW, KVIH, KVW, KVIL, 0, 1, 0,
		"bitmaps/keys/wrevup.xpm", "bitmaps/keys/wrevdown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 18 * KVW + KVIS, 0, KVMW, KVIH, 0, 1, 0,
		"bitmaps/keys/brevmup.xpm", "bitmaps/keys/brevmdown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 19 * KVW, KVIH, KVW, KVIL, 0, 1, 0,
		"bitmaps/keys/wrevup.xpm", "bitmaps/keys/wrevdown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 19 * KVW + KVIS, 0, KVMW, KVIH, 0, 1, 0,
		"bitmaps/keys/brevmup.xpm", "bitmaps/keys/brevmdown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 20 * KVW, KVIH, KVW, KVIL, 0, 1, 0,
		"bitmaps/keys/wrevup.xpm", "bitmaps/keys/wrevdown.xpm", BRIGHTON_NOSHADOW},

	{"", 2, 21 * KVW, KVIH, KVW, KVIL, 0, 1, 0,
		"bitmaps/keys/wrevup.xpm", "bitmaps/keys/wrevdown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 21 * KVW, 0, KVIW + 7, KVIH, 0, 1, 0,
		"bitmaps/keys/brevup.xpm", "bitmaps/keys/brevdown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 22 * KVW, KVIH, KVW, KVIL, 0, 1, 0,
		"bitmaps/keys/wrevup.xpm", "bitmaps/keys/wrevdown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 22 * KVW + KVIS, 0, KVMW, KVIH, 0, 1, 0,
		"bitmaps/keys/brevmup.xpm", "bitmaps/keys/brevmdown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 23 * KVW, KVIH, KVW, KVIL, 0, 1, 0,
		"bitmaps/keys/wrevup.xpm", "bitmaps/keys/wrevdown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 24 * KVW, KVIH, KVW, KVIL, 0, 1, 0,
		"bitmaps/keys/wrevup.xpm", "bitmaps/keys/wrevdown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 24 * KVW, 0, KVIW + 10, KVIH, 0, 1, 0,
		"bitmaps/keys/brevup.xpm", "bitmaps/keys/brevdown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 25 * KVW, KVIH, KVW, KVIL, 0, 1, 0,
		"bitmaps/keys/wrevup.xpm", "bitmaps/keys/wrevdown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 25 * KVW + KVIS, 0, KVMW, KVIH, 0, 1, 0,
		"bitmaps/keys/brevmup.xpm", "bitmaps/keys/brevmdown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 26 * KVW, KVIH, KVW, KVIL, 0, 1, 0,
		"bitmaps/keys/wrevup.xpm", "bitmaps/keys/wrevdown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 26 * KVW + KVIS, 0, KVMW, KVIH, 0, 1, 0,
		"bitmaps/keys/brevmup.xpm", "bitmaps/keys/brevmdown.xpm", BRIGHTON_NOSHADOW},
	{"", 2, 27 * KVW, KVIH, KVW, KVIL, 0, 1, 0,
		"bitmaps/keys/wrevup.xpm", "bitmaps/keys/wrevdown.xpm", BRIGHTON_NOSHADOW},
};

static brightonLocations mods[2] = {
	{"", 1, 290, 170, 97, 640, 0, 1, 0, 0, 0, BRIGHTON_CENTER|BRIGHTON_NOSHADOW},
	{"", 1, 655, 170, 97, 640, 0, 1, 0, 0, 0, BRIGHTON_NOSHADOW},
};

static int
modCallback(brightonWindow *win, int panel, int index, float value)
{
	guiSynth *synth = findSynth(global.synths, win);

//	printf("modCallback(%x, %i, %i, %f)\n", synth, panel, index, value);

	/*
	 * If this is controller 0 it is the frequency control, otherwise a 
	 * generic controller 1.
	 */
	if (index == 0)
		bristolMidiSendMsg(global.controlfd, synth->midichannel,
			BRISTOL_EVENT_PITCH, 0, (int) (value * C_RANGE_MIN_1));
	else {
		bristolMidiControl(global.controlfd, synth->midichannel,
			0, 1, ((int) (value * C_RANGE_MIN_1)) >> 7);
	}
}

static int
keyCallback(brightonWindow *win, int panel, int index, float value)
{
	guiSynth *synth = findSynth(global.synths, win);

//	printf("keycallback(%x, %i, %i, %f): %i %i\n", synth, panel, index, value,
//		synth->transpose, global.controlfd);

	/*
	 * Want to send a note event, on or off, for this index + transpose.
	 */
	if (value)
		bristolMidiSendMsg(global.controlfd, synth->midichannel,
			BRISTOL_EVENT_KEYON, 0, index + synth->transpose);
	else
		bristolMidiSendMsg(global.controlfd, synth->midichannel,
			BRISTOL_EVENT_KEYOFF, 0, index + synth->transpose);
}

#endif /* BRIGHTONKEYS_H */

