
/*
 *  Diverse Bristol audio routines.
 *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2006
 *
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */


#include <fcntl.h>
#include <string.h>

#include "brightonMini.h"

extern guimain global;

int
destroySynth(brightonWindow* win)
{
	guiSynth *synth = findSynth(global.synths, win);
	bristolMidiMsg msg;

printf("destroySynth(%i): %i\n", win, synth->sid);
	/*
	 * Since we registered two synths, we now need to remove the upper
	 * manual.
	 */
	bristolMidiSendMsg(global.controlfd, synth->sid, 127, 0,
		BRISTOL_EXIT_ALGO);

	if (bristolMidiRead(global.controlfd, &msg) < 0)
		printf("socket closed\n");
}

displayPanel(guiSynth *synth, char *text, int value, int panel, int index)
{
	brightonEvent event;
	char display[32];

	sprintf(display, "%s", text);
	event.type = BRIGHTON_MEM;
	event.m = (void *) &display[0];
	brightonParamChange(synth->win, panel, index, &event);
}

displayPanelText(guiSynth *synth, char *text, int value, int panel, int index)
{
	brightonEvent event;
	char display[32];

	sprintf(display, "%s: %i", text, value);
	event.type = BRIGHTON_MEM;
	event.m = (void *) &display[0];
	brightonParamChange(synth->win, panel, index, &event);
}

displayText(guiSynth *synth, char *text, int value, int index)
{
	brightonEvent event;
	char display[32];

	sprintf(display, "%s: %i", text, value);
	event.type = BRIGHTON_MEM;
	event.m = (void *) &display[0];
	brightonParamChange(synth->win, synth->panel, index, &event);
}

guiSynth *
findSynth(guiSynth *synth, brightonWindow* win)
{
//printf("findSynth(%x, %x)\n", synth, reference);
	if (synth == 0)
		return(0);

	if (synth->win == win)
		return(synth);

	return(findSynth(synth->next, win));
}

loadMemory(guiSynth *synth, char *algo, char *name, int location, int active,
int skip, int flags)
{
	brightonEvent event;
	char path[256];
	int i, fd, panel = 0, index = 0;
	memory tmem;

//printf("loadmemory %i: %s %x\n", location, synth->resources->name, flags);

	if ((name != 0) && (name[0] == '/'))
		sprintf(path, "%s", name);
	else {
//		printf("home is %s\n", global.home);

		sprintf(path, "%s/memory/%s/%s%i.mem",
			global.home, algo, algo, location);
	}

	if ((fd = open(path, O_RDONLY)) < 0)
	{
		return(-1);
	}

	if (flags & BRISTOL_STAT)
	{
		close(fd);
		return(0);
	}

	if (read(fd, &tmem.algo[0], 32) < 0)
	{
		printf("read failed on params\n");
		close(fd);
		return(-1);
	}
	if (read(fd, &tmem.name[0], 32) < 0)
	{
		printf("read failed on name\n");
		close(fd);
		return(-1);
	}
	if (read(fd, &tmem.count, 4 * sizeof(short)) < 0)
	{
		printf("read failed opts\n");
		close(fd);
		return(-1);
	}

	if (strcmp(&tmem.algo[0], algo) != 0)
	{
		printf("Attempt to read %s into %s algorithm\n",
			&tmem.algo[0], algo);

		if ((flags & BRISTOL_FORCE) == 0)
		{
			close(fd);
			return(-1);
		}
	}

	if (tmem.vers != 0)
	{
		/*
		 * Future revisions to the memory structures should put hooks in here
		 */
		printf("read failed: vers %i no supported\n", synth->mem.vers);
		close(fd);
		return(-1);
	}

	sprintf(&synth->mem.algo[0], "%s", &tmem.algo[0]);
	sprintf(&synth->mem.name[0], "%s", &tmem.name[0]);
	synth->mem.count = tmem.count;
	synth->mem.vers = tmem.vers;
	if (synth->mem.active != tmem.active)
		printf("Active count changed. Overriding\n");
//	synth->mem.active = tmem.active;

	if (read(fd, &synth->mem.param[skip], active * sizeof(float)) < 0)
		printf("read failed\n");

	close(fd);

	if (flags & BRISTOL_NOCALLS)
		return(0);

	synth->flags |= MEM_LOADING;
	/*
	 * We now have to call the GUI to configure all these values. The GUI
	 * will then call us back with the parameters to send to the synth.
	 */
	for (i = 0; i < synth->mem.active; i++)
	{
		event.type = BRIGHTON_FLOAT;
		event.value = synth->mem.param[i];
		/*
		 * We need to find which panel hosts this memory index. If we only
		 * have one panel, this is easy, it has all the controls. If we have
		 * multiple then we need to go through them all.
		 */
		if (synth->resources->nresources == 1)
			brightonParamChange(synth->win, synth->panel, i, &event);
		else {
			index = i;

			for (panel = 0; panel < synth->resources->nresources; panel++)
			{
				if ((index - synth->resources->resources[panel].ndevices) < 0)
				{
					/*
					 * If this is a text panel, skip it.
					 */
					if (synth->resources->resources[panel].devlocn[index].device
						== 3)
						continue;
					/*
					 * The next is odd, and could break a couple of the synth
					 * memories, but if the value that is in the newly loaded
					 * memory is outside of the configured range then we are
					 * not going to set the value.
					 */
					if ((synth->resources->resources[panel].devlocn[index].from
						< synth->resources->resources[panel].devlocn[index].to)
						&&
						((synth->resources->resources[panel].devlocn[index].from
							> event.value) ||
						(synth->resources->resources[panel].devlocn[index].to
							< event.value)))
						continue;
					brightonParamChange(synth->win, panel, index, &event);
					break;
				} else
					index -= synth->resources->resources[panel].ndevices;
			}
		}
	}
	synth->flags &= ~MEM_LOADING;

	return(0);
}

saveMemory(guiSynth *synth, char *algo, char *name, int location, int skip)
{
	char path[256];
	int fd;

//printf("savememory %i\n", location);

	if ((name != 0) && (name[0] == '/'))
	{
		sprintf(synth->mem.algo, "%s", algo);
		sprintf(path, "%s", name);
	} else {
		sprintf(path, "%s/memory/%s/%s%i.mem",
			global.home, algo, algo, location);
		sprintf(synth->mem.algo, "%s", algo);
		if (name == NULL)
			sprintf(synth->mem.name, "no name");
		else
			sprintf(synth->mem.name, "%s", name);
		printf("path is %s\n", path);
	}

	if ((fd = open(path, O_WRONLY|O_CREAT|O_TRUNC, 0770)) < 0)
		return(-1);

	if (write(fd, &synth->mem, sizeof(struct Memory) - sizeof(float *)) < 0)
		printf("write failed 1\n");
	if (write(fd, &synth->mem.param[skip], synth->mem.active * sizeof(float))
		< 0)
		printf("write failed 2\n");

	/*
	 * Save profile with controller registrations.
	 */
	brightonWriteConfiguration(synth->win, algo, synth->midichannel);

	close(fd);
}

initConnection(guimain *global, guiSynth *synth)
{
	bristolMidiMsg msg;
	int sid;

	forkengine(global);

	/*
	 * send a hello, with a voice count, then request starting of the
	 * synthtype. All of these will require an ACK, and we should wait here
	 * and read that ack before proceeding with each next step.
	 */
	bristolMidiSendMsg(global->controlfd, synth->midichannel,
//		127, 0, BRISTOL_HELLO|BRISTOL_VOICES|synth->voices);
		127, 0, BRISTOL_HELLO|synth->voices);
	if (bristolMidiRead(global->controlfd, &msg) != BRISTOL_OK)
		cleanupBristol();
	/*
	 * The returned value is the connection ID that we need to use for all
	 * further system conversations.
	 */
	sid = (msg.params.bristol.valueMSB << 7) + msg.params.bristol.valueLSB;

	bristolMidiSendMsg(global->controlfd, sid,
		127, 0, BRISTOL_MIDICHANNEL|synth->midichannel);
	if (bristolMidiRead(global->controlfd, &msg) != BRISTOL_OK)
		cleanupBristol();

	bristolMidiSendMsg(global->controlfd, sid,
		127, 0, BRISTOL_REQSTART);
	if (bristolMidiRead(global->controlfd, &msg) != BRISTOL_OK)
		cleanupBristol();

//printf("	initialising one connection of type %i\n", synth->synthtype);
	bristolMidiSendMsg(global->controlfd, sid,
		127, 0, BRISTOL_INIT_ALGO|synth->synthtype);
	if (bristolMidiRead(global->controlfd, &msg) != BRISTOL_OK)
		cleanupBristol();

	return(sid);
}

forkengine(guimain *global)
{
	int flags;

	if ((global->flags & BRIGHTON_NOENGINE) == 0)
	{
printf("starting engine\n");
		if ((global->enginePID = fork()) == 0)
		{
			execlp(BRISTOL_ENGINE, "engine", NULL);
			printf("Could not find bristol engine\n");
			_exit(-1);
		}
	}
	sleep(1);

	flags = BRISTOL_CONN_TCP|BRISTOL_DUPLEX|
		(global->flags &  BRISTOL_CONN_FORCE);

	if ((global->controlfd = bristolMidiOpen("localhost",
		flags, global->port, -1, NULL, NULL)) < 0)
	{
//printf("opening link to engine: %i\n", global->port);
		if ((global->controlfd = bristolMidiOpen("localhost",
			flags, global->port, -1, NULL, NULL)) < 0)
			cleanupBristol();
	} else
		printf("%s already active\n", BRISTOL_ENGINE);
}

